// ADCMRAID.js v2.2.3 #0  Mon Nov 09 2020 12:12:14 GMT-0800 (PST)

"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ADCLogger = (function () {
  function ADCLogger(message_dispatcher, use_native_log, os_name) {
    _classCallCheck(this, ADCLogger);

    this.message_dispatcher = message_dispatcher;
    this.use_native_log = use_native_log || false;
    this.os_name = os_name;
  }

  ADCLogger.prototype.trace = function trace(message, module) {};

  ADCLogger.prototype.trace_public = function trace_public(message, module) {

    this.log_public(message, "Log.public.trace", module);
  };

  ADCLogger.prototype.info = function info(message, module) {};

  ADCLogger.prototype.info_public = function info_public(message, module) {

    this.log_public(message, "Log.public.info", module);
  };

  ADCLogger.prototype.warning = function warning(message, module) {
    this.log(message, "Log.private.warning", module);
  };

  ADCLogger.prototype.warning_public = function warning_public(message, module) {
    this.log_public(message, "Log.public.warning", module);
  };

  ADCLogger.prototype.error = function error(message, module) {
    this.log(message, "Log.private.error", module);
  };

  ADCLogger.prototype.error_public = function error_public(message, module) {
    this.log_public(message, "Log.public.error", module);
  };

  ADCLogger.prototype.log = function log(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }
    this.message_dispatcher.queue_outbound_message({
      "m_origin": 1,
      "m_type": type,
      "m_target": 0,
      "message": "ADCAdUnit : " + message,
      "module": module
    });
  };

  ADCLogger.prototype.log_public = function log_public(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }
    this.message_dispatcher.queue_outbound_message({
      "m_origin": 1,
      "m_type": type,
      "m_target": 0,
      "message": "ADCAdUnit : " + message,
      "module": module
    });
  };

  ADCLogger.prototype.log_native = function log_native(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    message = "ADCAdUnit : " + message;

    switch (type) {
      case "Log.private.trace":
        logTrace(module, message);
        break;
      case "Log.public.trace":
        logTracePublic(module, message);
        break;
      case "Log.private.info":
        logInfo(module, message);
        break;
      case "Log.public.info":
        logInfoPublic(module, message);
        break;
      case "Log.private.warning":
        logWarn(module, message);
        break;
      case "Log.public.warning":
        logWarnPublic(module, message);
        break;
      case "Log.private.error":
        logError(module, message);
        break;
      case "Log.public.error":
        logErrorPublic(module, message);
        break;
    }
  };

  ADCLogger.prototype.log_native_adr = function log_native_adr(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    module = module.toString();
    message = "ADCAdUnit : " + message;

    switch (type) {
      case "Log.private.trace":
        logn(module, "3", message, "true");
        break;
      case "Log.public.trace":
        logn(module, "3", message, "false");
        break;
      case "Log.private.info":
        logn(module, "2", message, "true");
        break;
      case "Log.public.info":
        logn(module, "2", message, "false");
        break;
      case "Log.private.warning":
        logn(module, "1", message, "true");
        break;
      case "Log.public.warning":
        logn(module, "1", message, "false");
        break;
      case "Log.private.error":
        logn(module, "0", message, "true");
        break;
      case "Log.public.error":
        logn(module, "0", message, "false");
        break;
    }
  };

  return ADCLogger;
})();

var ADCMessageDispatcher = (function () {
  function ADCMessageDispatcher() {
    _classCallCheck(this, ADCMessageDispatcher);

    this.current_message_id = 0;
    this.message_handlers = {};
    this.message_reply_handlers = {};
    this.outbound = [];
    this.max_messages = 1024;
  }

  ADCMessageDispatcher.prototype.next_message_id = function next_message_id() {
    return this.current_message_id++;
  };

  ADCMessageDispatcher.prototype.queue_outbound_message = function queue_outbound_message(message, reply_handler) {
    if (message.m_id == undefined) {
      message.m_id = this.next_message_id();
    }

    var message_id = message.m_id;

    this.outbound.push(message);

    if (typeof reply_handler === "undefined" || reply_handler === null || typeof reply_handler !== "function") {
      return;
    }
    if (lookup(this.message_reply_handlers, message_id) === "undefined") {
      this.message_reply_handlers[message_id] = reply_handler;
    }

    return message_id;
  };

  ADCMessageDispatcher.prototype.register_message_handler = function register_message_handler(handler, message_type) {
    log_trace("Registering message handler for message type: " + message_type);

    if (lookup(this.message_handlers, message_type) === "undefined") {
      this.message_handlers[message_type] = [];
    }

    var handlers = lookup_array(this.message_handlers, message_type);
    for (var i = 0; i < handlers.length; ++i) {
      var first_handler = handlers[i];
      if (first_handler === handler) {
        log_warning("Tried to register multiple instances of the same message handler. Ignoring");
        return;
      }
    }

    this.message_handlers[message_type].push(handler);
  };

  ADCMessageDispatcher.prototype.unregister_reply_handler = function unregister_reply_handler(message_id) {
    log_trace("Unregistering message reply handler for message with id: " + message_id);
    delete this.message_reply_handlers[message_id];
  };

  ADCMessageDispatcher.prototype["package"] = function _package() {
    var how_many = this.outbound.length > this.max_messages ? this.max_messages : this.outbound.length;
    var outbound_messages = JSON.stringify(this.outbound.splice(0, how_many));

    return outbound_messages;
  };

  ADCMessageDispatcher.prototype.distribute = function distribute(inbound_messages) {
    inbound_messages = inbound_messages || [];

    for (var i = 0; i < inbound_messages.length; ++i) {
      var message = inbound_messages[i];
      var message_id = lookup_number(message, "m_id");
      var message_type = lookup_string(message, "m_type");

      if (message_type === "") {
        log_warning("Invalid message received. ignoring");
        continue;
      }

      if (message_id === -1) {
        log_warning("Invalid message received. ignoring");
        continue;
      }

      if (message_type === "reply") {
        if (lookup(this.message_reply_handlers, message_id) === "undefined") {
          log_info("No reply handler registered for message with id: " + message_id);
          continue;
        }

        var handler = this.message_reply_handlers[message_id];
        handler(message);
        this.unregister_reply_handler(message_id);
      } else {

        if (lookup_array(this.message_handlers, message_type).length === 0) {
          log_info("No handler has been registered to message type: " + message_type);
          continue;
        }

        var handlers = lookup_array(this.message_handlers, message_type);
        for (var j = 0; j < handlers.length; ++j) {
          var handler = handlers[j];
          handler(message);
        }
      }
    }
  };

  return ADCMessageDispatcher;
})();

var log_trace = function log_trace(message, module) {
  $.log.trace(message, module);
};

var log_trace_public = function log_trace_public(message, module) {
  $.log.trace_public(message, module);
};

var log_info = function log_info(message, module) {
  $.log.info(message, module);
};

var log_info_public = function log_info_public(message, module) {
  $.log.info_public(message, module);
};

var log_warning = function log_warning(message, module) {
  $.log.warning(message, module);
};

var log_warning_public = function log_warning_public(message, module) {
  $.log.warning_public(message, module);
};

var log_error = function log_error(message, module) {
  $.log.error(message, module);
};

var log_error_public = function log_error_public(message, module) {
  $.log.error_public(message, module);
};

var log = function log(message, type, module) {
  $.log.log(message, type, module);
};

var log_public = function log_public(message, type, module) {
  $.log.log_public(message, type, module);
};

var lookup = function lookup(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return "undefined";
  }

  if (typeof key === "undefined") {
    return "undefined";
  }

  if (dict[key] === 0) {
    return 0;
  }

  return dict[key] || "undefined";
};

var lookup_string = function lookup_string(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    log_trace("dict is undefined in lookup_string");
    return "";
  }

  if (typeof key === "undefined") {
    log_trace("key is undefined in lookup_string");
    return "";
  }

  var maybe_string = dict[key];
  if (typeof maybe_string === "string") {
    return maybe_string;
  }

  log_trace("return default value of empty string in lookup_string");
  return "";
};

var lookup_boolean = function lookup_boolean(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return false;
  }

  if (typeof key === "undefined") {
    return false;
  }

  var maybe_boolean = dict[key];
  if (typeof maybe_boolean === "boolean") {
    return maybe_boolean;
  } else if (typeof maybe_boolean === "number") {
    if (maybe_boolean != 0) {
      return true;
    } else {
      return false;
    }
  }

  return false;
};

var lookup_number = function lookup_number(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return -1;
  }

  if (typeof key === "undefined") {
    return -1;
  }

  var maybe_number = dict[key];
  if (typeof maybe_number === "number") {
    return maybe_number;
  }

  return -1;
};

var lookup_array = function lookup_array(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return [];
  }

  if (typeof key === "undefined") {
    return [];
  }

  var maybe_array = dict[key];
  if (maybe_array instanceof Array) {
    return maybe_array;
  }

  return [];
};

var lookup_default = function lookup_default(dict, key, _default) {

  if (typeof dict === "undefined" || dict === null) {
    return _default;
  }

  if (typeof dict[key] === "undefined") {
    return _default;
  }

  return dict[key];
};

var parse_json_with_check = function parse_json_with_check(str) {
  if (typeof str === "undefined") {
    log_warning("Tried to parse an undefined JSON string");
    return false;
  }

  if (typeof str !== "string") {
    log_warning("Tried to parse a JSON string that is not a string: " + JSON.stringify(str));
    return false;
  }

  try {
    var result = JSON.parse(str);
    return result;
  } catch (e) {
    log_warning("ADCController encountered a JSON parsing error with string: " + str);
  }

  return false;
};

var replace_keys = function replace_keys(dict, mapping) {
  for (var key in dict) {
    var new_key = lookup_string(mapping, key);
    if (new_key !== "") {
      dict[new_key] = dict[key];
      delete dict[key];
    }
  }
  return dict;
};

var merge_dictionaries = function merge_dictionaries(dict1, dict2) {
  var new_dict = {};

  for (var key in dict1) {
    new_dict[key] = dict1[key];
  }

  for (var key in dict2) {
    new_dict[key] = dict2[key];
  }

  return new_dict;
};

function ADC_get_sentinel() {
  return 37;
}

var $ = {
  log: null,
  module_id: -1,
  replay: false,
  reward: false,
  ad_unit_module_id: null,
  preload_store_requests: []
};

var ADC3_init = function ADC3_init(module_id, ad_unit_config) {

  $.message_dispatcher = new ADCMessageDispatcher();

  $.log = new ADCLogger($.message_dispatcher, true, lookup_string(ad_unit_config, "os_name"));
  log_info("mraid ad unit config: " + JSON.stringify(ad_unit_config));
  $.module_id = module_id;
  if (typeof ad_unit_config !== "undefined") {
    $.ad_session_id = ad_unit_config.ad_session_id;
    $.reward = lookup_boolean(ad_unit_config, "reward");
    $.ad_unit_module_id = lookup_number(ad_unit_config, "ad_unit_module_id");
  }

  if ($.preload_store_requests) {

    var length = $.preload_store_requests.length;
    if (length > 0) {
      for (var i = 0; i < length; ++i) {
        var request = $.preload_store_requests.shift();
        $.message_dispatcher.queue_outbound_message({
          "m_origin": $.module_id,
          "m_target": 0,
          "m_type": "System.preload_store",
          "product_data": request || {}
        }, function (message_reply) {
          mraid.fireEvent(mraid.EVENTS.STORE_LOAD, lookup_boolean(message_reply, "success"), request["id"] || "");
        });
      }
    }
  }

  $.ad_unit_module_id = lookup_number(ad_unit_config, "ad_unit_module_id");

  $.message_dispatcher.register_message_handler(function (message) {
    log_info("MRAID viewable change detected");
    mraid.fireEvent("viewableChange", lookup_boolean(message, "visible"));
  }, "MRAID.viewable_change");

  $.message_dispatcher.register_message_handler(function (message) {
    log_info("MRAID detecting SessionInfo.on_pause message");
  }, "SessionInfo.on_pause");

  $.message_dispatcher.register_message_handler(function (message) {
    log_info("MRAID detecting SessionInfo.on_resume message");
  }, "SessionInfo.on_resume");

  $.message_dispatcher.register_message_handler(function (message) {
    $.replay = true;
  }, "MRAID.on_replay");
};

var ADC3_update = function ADC3_update(inbound_messages) {
  $.message_dispatcher.distribute(inbound_messages);
  var outbound = $.message_dispatcher["package"]();
  return outbound;
};

(function (global) {

  var mraid = {};
  var bridge = global.adc_bridge = {};

  bridge.adc_version = "0.0";

  bridge.os_version = "0.0";

  bridge.os_name = "";

  if (typeof ADC_DEVICE_INFO !== "undefined" && ADC_DEVICE_INFO != null) {
    bridge.os_name = ADC_DEVICE_INFO.platform;
    ADC_DEVICE_INFO.os_name = ADC_DEVICE_INFO.platform;
  }

  var features = mraid.features = {
    tel: false,
    sms: false,
    calendar: false,
    mail: false,
    storePicture: false,
    inlineVideo: false
  };

  var VERSION = mraid.VERSION = "2.0";

  var STATES = mraid.STATES = {
    LOADING: "loading",
    DEFAULT: "default",
    EXPANDED: "expanded",
    HIDDEN: "hidden",

    RESIZED: "resized"
  };

  var EVENTS = mraid.EVENTS = {
    ERROR: "error",
    READY: "ready",
    STATECHANGE: "stateChange",
    VIEWABLECHANGE: "viewableChange",
    SIZECHANGE: "sizeChange",

    INFO: "info",
    APP_PRESENCE: "appPresence",
    SOCIAL_PRESENCE: "socialPresence",
    SOCIAL_POST: "socialPost",
    STORE_LOAD: "storeLoad",
    STORE_UNLOAD: "storeUnload",

    EXPOSURECHANGE: "exposureChange"
  };

  var ADC_EVENTS = mraid.ADC_EVENTS = {
    INFO: "info",
    DOWNLOAD: "download",
    CONTINUE: "continue"
  };

  var PLACEMENT_TYPES = mraid.PLACEMENT_TYPES = {
    UNKNOWN: "unknown",
    INLINE: "inline",
    INTERSTITIAL: "interstitial"
  };

  var ADC_SOCIAL_SERVICES = mraid.ADC_SOCIAL_SERVICES = {
    FACEBOOK: "facebook",
    TWITTER: "twitter",
    SINA_WEIBO: "sina weibo"
  };

  var _expandProperties = {
    width: -1,
    height: -1,
    useCustomClose: false,
    isModal: true,
    lockOrientation: false
  };

  var hasSetCustomSize = false;
  var hasSetCustomClose = false;
  var listeners = mraid.listeners = {};

  var _state = STATES.LOADING;
  var isViewable = false;
  var _screenSize = { width: -1, height: -1 };
  var _placementType = PLACEMENT_TYPES.INTERSTITIAL;

  var storeAttr = function storeAttr(data) {
    if (ADC_DEVICE_INFO.platform.toLowerCase() == "ios" && ADC_DEVICE_INFO.attr && ADC_DEVICE_INFO.attr.id) {
      return ADC_DEVICE_INFO.attr;
    }
    return data;
  };

  var parseNumber = function parseNumber(num) {
    var parsed = typeof num == 'number' ? num : parseInt(num, 10);
    if (isNaN(parsed)) {
      return 0;
    }
    return parsed;
  };

  var parseVersion = function parseVersion(version) {
    var parts = ("" + version).split('.');
    var output = [0, 0, 0, 0];
    for (var i = 0; i < 4; ++i) {
      output[i] = parts.length > i ? parseNumber(parts[i]) : 0;
    }
    return output;
  };

  var compareVersion = function compareVersion(v1, v2) {
    var vp1 = parseVersion(v1);
    var vp2 = parseVersion(v2);
    for (var i = 0; i < 4; ++i) {
      var val1 = vp1[i];
      var val2 = vp2[i];
      if (val1 != val2) {
        return val1 > val2 ? 1 : -1;
      }
    }
    return 0;
  };

  var sanitizeProductData = function sanitizeProductData(data) {
    if (!data.product_id && data.id) {
      data.product_id = data.id;
    }
    if (!data.id && data.product_id) {
      data.id = data.product_id;
    }
    data.product_id = "" + data.product_id;
    data.id = compareVersion(ADC_DEVICE_INFO.adc_version, '3.3.6') < 0 ? "" + data.id : parseNumber(data.id);
    return data;
  };

  mraid.addEventListener = function (event, listener) {
    var eventListeners;
    listeners[event] = listeners[event] || [];
    eventListeners = listeners[event];

    for (var l in eventListeners) {

      if (listener === l) return;
    }

    eventListeners.push(listener);
    if (event == "exposureChange") {
      var initial_exposure = 0;

      if ($.ad_unit) {
        initial_exposure = $.ad_unit.percent_viewable;
      }

      listener.call(null, initial_exposure);
    }
  };

  mraid.removeEventListener = function (event, listener) {
    if (listeners.hasOwnProperty(event)) {
      var eventListeners = listeners[event];
      if (eventListeners) {
        var idx = eventListeners.indexOf(listener);
        if (idx !== -1) {
          eventListeners.splice(idx, 1);
        }
      }
    }
  };

  mraid.fireEvent = function (type) {
    var ls = listeners[type];
    if (ls) {
      var args = Array.prototype.slice.call(arguments);
      args.shift();
      var l = ls.length;
      for (var i = 0; i < l; i++) {
        ls[i].apply(null, args);
      }
    }
  };
  var changeHandlers = {
    state: function state(val) {
      log_info("Changing mraid state to " + val);
      if (_state === STATES.LOADING) {
        log_info("Native SDK initialized.");
      }
      _state = val;
      log_info("Set state to " + JSON.stringify(val));
    },

    viewable: function viewable(val) {
      log_info("Changing mraid viewable to " + val);
      isViewable = val;

      log_info("Set isViewable to " + JSON.stringify(val));
    },

    placementType: function placementType(val) {
      log_info("Changing mraid placementType to " + val);
      _placementType = val;
      log_info("Set placementType to " + JSON.stringify(val));
    },

    screenSize: function screenSize(val) {
      for (var key in val) {
        if (val.hasOwnProperty(key)) {
          _screenSize[key] = val[key];
          log_info("screen " + key + " is now " + val[key]);
        }
      }

      if (!hasSetCustomSize) {
        _expandProperties["width"] = _screenSize["width"];
        _expandProperties["height"] = _screenSize["height"];
      }

      log_info("Set screenSize to " + JSON.stringify(val));
    },

    sizeChange: function sizeChange(val) {},

    expandProperties: function expandProperties(val) {
      for (var key in val) {
        if (val.hasOwnProperty(key)) _expandProperties[key] = val[key];
      }
      log_info("Merging expandProperties with " + JSON.stringify(val));
    }
  };

  var validate = function validate(obj, validators, action, merge) {
    if (!merge) {

      if (obj === null) {
        log_error("mraid.expandProperties: Required object not provided.");
        return false;
      } else {
        for (var i in validators) {
          if (validators.hasOwnProperty(i) && obj[i] === undefined) {
            log_error("mraid.expandProperties: Object is missing required property: " + i + ".");
            return false;
          }
        }
      }
    }

    for (var prop in obj) {
      var validator = validators[prop];
      var value = obj[prop];
      if (validator && !validator(value)) {

        log_error("mraid.expandProperties: Value of property " + prop + " is invalid.");
        return false;
      }
    }
    return true;
  };

  var expandPropertyValidators = {
    width: function width(v) {
      return !isNaN(v) && v >= 0;
    },
    height: function height(v) {
      return !isNaN(v) && v >= 0;
    },
    useCustomClose: function useCustomClose(v) {
      return typeof v === "boolean";
    },
    lockOrientation: function lockOrientation(v) {
      return typeof v === "boolean";
    }
  };
  mraid.close = function () {
    if (_state === STATES.HIDDEN) {
      log_error("mraid.close: Ad cannot be closed when it is already hidden.");
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_origin": $.module_id,
        "m_target": 0,
        "m_type": "System.close",
        "ad_session_id": $.ad_session_id
      });
    }
  };

  mraid.expand = function (URL) {
    if (_state !== STATES.DEFAULT) {
      log_error("mraid.expand: Can not expand, because state is not in the default state");
    } else {
      var args = ["expand"];

      if (hasSetCustomClose) {
        args = args.concat(["shouldUseCustomClose", _expandProperties.useCustomClose ? "true" : "false"]);
      }

      if (hasSetCustomSize) {
        if (_expandProperties.width >= 0 && _expandProperties.height >= 0) {
          args = args.concat(["w", _expandProperties.width, "h", _expandProperties.height]);
        }
      }

      if (typeof _expandProperties.lockOrientation !== "undefined") {
        args = args.concat(["lockOrientation", _expandProperties.lockOrientation]);
      }

      if (URL) {
        args = args.concat(["url", URL]);
      }
    }
  };

  mraid.getExpandProperties = function () {
    var properties = {
      width: _expandProperties.width,
      height: _expandProperties.height,
      useCustomClose: _expandProperties.useCustomClose,
      isModal: _expandProperties.isModal
    };

    return properties;
  };

  mraid.getPlacementType = function () {
    return _placementType;
  };

  mraid.getState = function () {
    return _state;
  };

  mraid.getVersion = function () {
    return mraid.VERSION;
  };

  mraid.isViewable = function () {
    return isViewable;
  };

  mraid.open = function (URL) {
    mraid.sendHTMLInteraction();

    if (ADC_DEVICE_INFO.click_through !== null && typeof ADC_DEVICE_INFO.click_through !== "undefined" && ADC_DEVICE_INFO.click_through !== "") {
      URL = ADC_DEVICE_INFO.click_through;
    }

    if (URL.indexOf("adcvideo") == 0) {
      $.message_dispatcher.queue_outbound_message({
        "m_origin": $.module_id,
        "m_target": 1,
        "m_type": "AdUnit.self_hosted_video",
        "url": URL.replace("adcvideo", "http"),
        "module_id": $.ad_unit_module_id
      });
      return;
    }

    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.open_browser",
      "url": URL,
      "external": false,
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.open_browser succeeded");
      } else {
        log_info("System.open_browser failed");
      }
    });
  };

  mraid.setExpandProperties = function (properties) {
    if (validate(properties, expandPropertyValidators, "setExpandProperties", true)) {
      if (properties.hasOwnProperty("width") || properties.hasOwnProperty("height")) {
        hasSetCustomSize = true;
      }

      if (properties.hasOwnProperty("useCustomClose")) hasSetCustomClose = true;

      var desiredProperties = ["width", "height", "useCustomClose", "lockOrientation"];
      var length = desiredProperties.length;
      for (var i = 0; i < length; i++) {
        var propname = desiredProperties[i];
        if (properties.hasOwnProperty(propname)) _expandProperties[propname] = properties[propname];
      }
    }
  };

  mraid.useCustomClose = function (shouldUseCustomClose) {
    _expandProperties.useCustomClose = shouldUseCustomClose;
    hasSetCustomClose = true;
  };

  mraid.getMaxSize = mraid.getScreenSize = function () {
    return _screenSize;
  };

  mraid.getScreenSize = function () {
    return { width: window.innerWidth, height: window.innerHeight };
  };

  mraid.getCurrentPosition = function () {

    return { x: 0, y: 0, width: window.innerWidth, height: window.innerHeight };
  };

  mraid.getDefaultPosition = function () {
    return { x: 0, y: 0, width: _screenSize["width"], height: _screenSize["height"] };
  };

  mraid.setFeatures = function (properties) {
    features = properties;
  };

  mraid.supports = function (feature) {
    return !!features[feature];
  };

  mraid.createCalendarEvent = function (properties) {

    mraid.sendHTMLInteraction();
    properties = properties || {};

    properties.url = properties.url || properties.URL || "";

    properties.recurrence = properties.recurrence || "";

    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.create_calendar_event",
      "params": properties,
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.create_calendar_event succeeded");
      } else {
        log_info("System.create_calendar_event failed");
      }
    });
  };
  mraid.sendADCEvent = function (params, category, name) {
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 1,
      "m_type": "AdUnit.on_event",
      "ad_session_id": $.ad_session_id,
      "event_type": params.toLowerCase()
    }, function (message_reply) {});
  };

  mraid.checkSocialPresence = function (service) {
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.check_social_presence",
      "service": service
    }, function (message_reply) {
      log_info("ADCMRAIDMessageHandler: " + lookup_string(message_reply, "service") + " -- " + lookup_boolean(message_reply, "result"));
      mraid.fireEvent(mraid.EVENTS.SOCIAL_PRESENCE, lookup_string(message_reply, "service"), lookup_boolean(message_reply, "result"));
    });
  };

  mraid.makeInAppPurchase = function (product, quantity) {
    mraid.sendHTMLInteraction();

    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.make_in_app_purchase",
      "ad_session_id": $.ad_session_id,
      "product": product
    });
  };

  mraid.vibrate = function (time) {
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.vibrate",
      'length_ms': time
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.vibrate succeeded");
      } else {
        log_info("System.vibrate failed");
      }
    });
  };

  mraid.saveScreenshot = function () {
    mraid.sendHTMLInteraction();
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.save_screenshot"
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.save_screenshot succeeded");
      } else {
        log_info("System.save_screenshot failed");
      }
    });
  };

  mraid.socialPost = function (service, text, url) {
    mraid.sendHTMLInteraction();
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.social_post",
      "service": service,
      "text": text || "",
      "url": url || "",
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      mraid.fireEvent(mraid.EVENTS.SOCIAL_POST, lookup_string(message_reply, "service"), lookup_boolean(message_reply, "result"));
    });
  };

  mraid.sendADCCustomEvent = function (event_type) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_custom_event",
      "m_origin": $.module_id,
      "m_target": 1,
      "ad_session_id": $.ad_session_id,
      "replay": $.replay,
      "reward": $.reward,
      "event_type": event_type
    }, function (message_reply) {});
  };

  mraid.sendHTMLInteraction = function () {
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 1,
      "m_type": "AdUnit.on_html5_interaction",
      "ad_session_id": $.ad_session_id,
      "replay": $.replay,
      "reward": $.reward,
      "ad_unit_module_id": $.ad_unit_module_id
    });
  };

  mraid.openStore = function (item) {
    var result = item;

    if (typeof item === "string") {
      result = {};
      result.product_id = item;

      if (ADC_DEVICE_INFO.platform.toLowerCase() == "ios") {
        var re1 = /id\d+/;
        var re2 = /id=\d+/;

        var start = -1;
        var end = -1;

        if ((start = item.search(re1)) > 0) {
          start += 2;
          end = item.indexOf('?');
        } else if ((start = item.search(re2)) > 0) {
          start += 3;
          end = item.indexOf('&');
        }

        end = end > 0 ? end : item.length;
        result.product_id = item.substring(start, end);
      } else {

        result.product_id = item;
      }
    }

    result = sanitizeProductData(storeAttr(result));

    mraid.sendHTMLInteraction();

    if (ADC_DEVICE_INFO.click_through !== null && typeof ADC_DEVICE_INFO.click_through !== "undefined" && ADC_DEVICE_INFO.click_through !== "") {
      $.message_dispatcher.queue_outbound_message({
        "m_origin": $.module_id,
        "m_target": 0,
        "m_type": "System.open_browser",
        "url": ADC_DEVICE_INFO.click_through,
        "external": false,
        "ad_session_id": $.ad_session_id
      });
      return;
    }

    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.open_store",
      "m_origin": $.module_id,
      "m_target": 0,
      "product_id": result.product_id,
      "product_data": result,
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.open_store succeeded");
      } else {
        log_info("System.open_store failed");
      }
    });
  };

  mraid.checkAppPresence = function (handle) {
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.check_app_presence",
      "name": handle
    }, function (message_reply) {
      mraid.fireEvent(mraid.EVENTS.APP_PRESENCE, lookup_string(message_reply, "name"), lookup_boolean(message_reply, "result"));
    });
  };

  mraid.launchApp = function (handle) {
    mraid.sendHTMLInteraction();
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.launch_app",
      "handle": handle,
      "deep_link": false,
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.launch_app succeeded");
      } else {
        log_info("System.launch_app failed");
      }
    });
  };

  mraid.getSDKVersion = function () {
    return ADC_DEVICE_INFO.adc_version;
  };

  mraid.getOSVersion = function () {
    return ADC_DEVICE_INFO.os_version;
  };

  mraid.getOSName = function () {
    return ADC_DEVICE_INFO.platform;
  };

  mraid.sendSMS = function (properties) {
    mraid.sendHTMLInteraction();
    properties = properties || {};
    var to = properties.to || "";
    if (!(to instanceof Array)) {
      to = [to];
    }

    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.sms",
      "m_origin": $.module_id,
      "m_target": 0,
      "recipients": to || "",
      "body": properties.body || "",
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.sms succeeded");
      } else {
        log_info("System.sms failed");
      }
    });
  };

  mraid.sendMail = function (properties) {
    mraid.sendHTMLInteraction();
    properties = properties || {};
    var to = properties.to || "";
    if (!(to instanceof Array)) {
      to = [to];
    }

    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.mail",
      "m_origin": $.module_id,
      "m_target": 0,
      "recipients": to,
      "subject": properties.subject || "",
      "body": properties.body || "",
      "html": properties.html || false,
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.mail succeeded");
      } else {
        log_info("System.mail failed");
      }
    });
  };

  mraid.placeCall = function (number) {
    mraid.sendHTMLInteraction();
    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.telephone",
      "phone_number": number + "",
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      var success = lookup_boolean(message_reply, "success");
      if (success) {
        log_info("System.telephone succeeded");
      } else {
        log_info("System.telephone failed");
      }
    });
  };

  mraid.preloadStore = function (data) {
    var prod_data = sanitizeProductData(storeAttr(data));

    if ($.module_id === -1) {
      $.preload_store_requests.push(prod_data);
      return;
    }

    $.message_dispatcher.queue_outbound_message({
      "m_origin": $.module_id,
      "m_target": 0,
      "m_type": "System.preload_store",
      "product_id": (prod_data || {}).product_id,
      "product_data": prod_data || {},
      "ad_session_id": $.ad_session_id
    }, function (message_reply) {
      mraid.fireEvent(mraid.EVENTS.STORE_LOAD, lookup_boolean(message_reply, "success"), prod_data["id"] || "");
    });
  };

  mraid.getVendor = function () {
    return "AdColony";
  };

  mraid.getVendorVersion = function () {
    return ADC_DEVICE_INFO.adc_version;
  };

  mraid.sendADCEventPageShow = function (params) {
    mraid.sendADCEvent(params, "page", "show");
  };
  mraid.sendADCEventPageClose = function (params) {
    mraid.sendADCEvent(params, "page", "close");
  };
  mraid.sendADCEventPage = function (params) {
    mraid.sendADCEvent(params, "page");
  };

  mraid.sendADCEventClickWeb = function (params) {
    mraid.sendADCEvent(params, "click", "web");
  };
  mraid.sendADCEventClickDownload = function (params) {
    mraid.sendADCEvent(params, "click", "download");
  };
  mraid.sendADCEventClickMaps = function (params) {
    mraid.sendADCEvent(params, "click", "maps");
  };
  mraid.sendADCEventClickCalendar = function (params) {
    mraid.sendADCEvent(params, "click", "calendar");
  };
  mraid.sendADCEventClickYouTube = function (params) {
    mraid.sendADCEvent(params, "click", "youtube");
  };
  mraid.sendADCEventClickTwitter = function (params) {
    mraid.sendADCEvent(params, "click", "twitter");
  };
  mraid.sendADCEventClickFacebook = function (params) {
    mraid.sendADCEvent(params, "click", "facebook");
  };
  mraid.sendADCEventClickShowtimes = function (params) {
    mraid.sendADCEvent(params, "click", "showtimes");
  };
  mraid.sendADCEventClickSearch = function (params) {
    mraid.sendADCEvent(params, "click", "search");
  };
  mraid.sendADCEventClick = function (params) {
    mraid.sendADCEvent(params, "click");
  };

  mraid.sendADCEventSocialTwitter = function (params) {
    mraid.sendADCEvent(params, "social", "twitter");
  };
  mraid.sendADCEventSocialFacebook = function (params) {
    mraid.sendADCEvent(params, "social", "facebook");
  };
  mraid.sendADCEventSocialYouTube = function (params) {
    mraid.sendADCEvent(params, "social", "youtube");
  };
  mraid.sendADCEventSocialPinterest = function (params) {
    mraid.sendADCEvent(params, "social", "pinterest");
  };
  mraid.sendADCEventSocialInstagram = function (params) {
    mraid.sendADCEvent(params, "social", "instagram");
  };
  mraid.sendADCEventSocialGooglePlus = function (params) {
    mraid.sendADCEvent(params, "social", "googleplus");
  };
  mraid.sendADCEventSocialTumblr = function (params) {
    mraid.sendADCEvent(params, "social", "tumblr");
  };
  mraid.sendADCEventSocialRSS = function (params) {
    mraid.sendADCEvent(params, "social", "rss");
  };
  mraid.sendADCEventSocial = function (params) {
    mraid.sendADCEvent(params, "social");
  };

  mraid.sendADCEventMediaVideo = function (params) {
    mraid.sendADCEvent(params, "media", "video");
  };
  mraid.sendADCEventMediaPhoto = function (params) {
    mraid.sendADCEvent(params, "media", "photo");
  };

  mraid.sendADCEventGallerySwipe = function (params) {
    mraid.sendADCEvent(params, "gallery", "swipe");
  };
  mraid.sendADCEventFrameScroll = function (params) {
    mraid.sendADCEvent(params, "frame", "scroll");
  };
  mraid.sendADCEventDeviceMotionShake = function (params) {
    mraid.sendADCEvent(params, "motion", "shake");
  };
  mraid.sendADCEventDeviceMotionTilt = function (params) {
    mraid.sendADCEvent(params, "motion", "tilt");
  };
  mraid.sendADCEventDeviceMotionRotate = function (params) {
    mraid.sendADCEvent(params, "motion", "rotate");
  };
  mraid.sendADCEventTouchSwipe = function (params) {
    mraid.sendADCEvent(params, "touch", "swipe");
  };
  mraid.sendADCEventTouchFlick = function (params) {
    mraid.sendADCEvent(params, "touch", "flick");
  };
  mraid.sendADCEventTouchDragDrop = function (params) {
    mraid.sendADCEvent(params, "touch", "dragdrop");
  };
  mraid.sendADCEventTouchMultiTouch = function (params) {
    mraid.sendADCEvent(params, "touch", "multitouch");
  };
  mraid.sendADCEventTouchPinch = function (params) {
    mraid.sendADCEvent(params, "touch", "pinch");
  };
  mraid.sendADCEventTouchWipeAway = function (params) {
    mraid.sendADCEvent(params, "touch", "wipeaway");
  };
  mraid.sendADCEventTouchDraw = function (params) {
    mraid.sendADCEvent(params, "touch", "draw");
  };

  global.mraid = mraid;

  global.ADC_DEVICE_INFO = ADC_DEVICE_INFO;
})(typeof window === "undefined" ? undefined : window);
