"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ADCLogger = (function () {
  function ADCLogger(message_dispatcher, use_native_log, os_name) {
    _classCallCheck(this, ADCLogger);

    this.message_dispatcher = message_dispatcher;
    this.use_native_log = use_native_log || false;
    this.os_name = os_name;
  }

  ADCLogger.prototype.trace = function trace(message, module) {};

  ADCLogger.prototype.trace_public = function trace_public(message, module) {

    this.log_public(message, "Log.public.trace", module);
  };

  ADCLogger.prototype.info = function info(message, module) {};

  ADCLogger.prototype.info_public = function info_public(message, module) {

    this.log_public(message, "Log.public.info", module);
  };

  ADCLogger.prototype.warning = function warning(message, module) {
    this.log(message, "Log.private.warning", module);
  };

  ADCLogger.prototype.warning_public = function warning_public(message, module) {
    this.log_public(message, "Log.public.warning", module);
  };

  ADCLogger.prototype.error = function error(message, module) {
    this.log(message, "Log.private.error", module);
  };

  ADCLogger.prototype.error_public = function error_public(message, module) {
    this.log_public(message, "Log.public.error", module);
  };

  ADCLogger.prototype.log = function log(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    if (this.os_name === "ios") {
      this.log_native(message, type, module);
    } else {
      this.log_native_adr(message, type, module);
    }
  };

  ADCLogger.prototype.log_public = function log_public(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    if (this.os_name === "ios") {
      this.log_native(message, type, module);
    } else {
      this.log_native_adr(message, type, module);
    }
  };

  ADCLogger.prototype.log_native = function log_native(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    message = "ADCAdUnit : " + message;

    switch (type) {
      case "Log.private.trace":
        logTrace(module, message);
        break;
      case "Log.public.trace":
        logTracePublic(module, message);
        break;
      case "Log.private.info":
        logInfo(module, message);
        break;
      case "Log.public.info":
        logInfoPublic(module, message);
        break;
      case "Log.private.warning":
        logWarn(module, message);
        break;
      case "Log.public.warning":
        logWarnPublic(module, message);
        break;
      case "Log.private.error":
        logError(module, message);
        break;
      case "Log.public.error":
        logErrorPublic(module, message);
        break;
    }
  };

  ADCLogger.prototype.log_native_adr = function log_native_adr(message, type, module) {
    if (typeof module === "undefined") {
      module = 2;
    }

    module = module.toString();
    message = "ADCAdUnit : " + message;

    switch (type) {
      case "Log.private.trace":
        logn(module, "3", message, "true");
        break;
      case "Log.public.trace":
        logn(module, "3", message, "false");
        break;
      case "Log.private.info":
        logn(module, "2", message, "true");
        break;
      case "Log.public.info":
        logn(module, "2", message, "false");
        break;
      case "Log.private.warning":
        logn(module, "1", message, "true");
        break;
      case "Log.public.warning":
        logn(module, "1", message, "false");
        break;
      case "Log.private.error":
        logn(module, "0", message, "true");
        break;
      case "Log.public.error":
        logn(module, "0", message, "false");
        break;
    }
  };

  return ADCLogger;
})();

var log_trace = function log_trace(message, module) {
  $.log.trace(message, module);
};

var log_trace_public = function log_trace_public(message, module) {
  $.log.trace_public(message, module);
};

var log_info = function log_info(message, module) {
  $.log.info(message, module);
};

var log_info_public = function log_info_public(message, module) {
  $.log.info_public(message, module);
};

var log_warning = function log_warning(message, module) {
  $.log.warning(message, module);
};

var log_warning_public = function log_warning_public(message, module) {
  $.log.warning_public(message, module);
};

var log_error = function log_error(message, module) {
  $.log.error(message, module);
};

var log_error_public = function log_error_public(message, module) {
  $.log.error_public(message, module);
};

var log = function log(message, type, module) {
  $.log.log(message, type, module);
};

var log_public = function log_public(message, type, module) {
  $.log.log_public(message, type, module);
};

var lookup = function lookup(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return "undefined";
  }

  if (typeof key === "undefined") {
    return "undefined";
  }

  if (dict[key] === 0) {
    return 0;
  }

  return dict[key] || "undefined";
};

var lookup_string = function lookup_string(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    log_trace("dict is undefined in lookup_string");
    return "";
  }

  if (typeof key === "undefined") {
    log_trace("key is undefined in lookup_string");
    return "";
  }

  var maybe_string = dict[key];
  if (typeof maybe_string === "string") {
    return maybe_string;
  }

  log_trace("return default value of empty string in lookup_string");
  return "";
};

var lookup_boolean = function lookup_boolean(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return false;
  }

  if (typeof key === "undefined") {
    return false;
  }

  var maybe_boolean = dict[key];
  if (typeof maybe_boolean === "boolean") {
    return maybe_boolean;
  } else if (typeof maybe_boolean === "number") {
    if (maybe_boolean != 0) {
      return true;
    } else {
      return false;
    }
  }

  return false;
};

var lookup_number = function lookup_number(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return -1;
  }

  if (typeof key === "undefined") {
    return -1;
  }

  var maybe_number = dict[key];
  if (typeof maybe_number === "number") {
    return maybe_number;
  }

  return -1;
};

var lookup_array = function lookup_array(dict, key) {

  if (typeof dict === "undefined" || dict === null) {
    return [];
  }

  if (typeof key === "undefined") {
    return [];
  }

  var maybe_array = dict[key];
  if (maybe_array instanceof Array) {
    return maybe_array;
  }

  return [];
};

var lookup_default = function lookup_default(dict, key, _default) {

  if (typeof dict === "undefined" || dict === null) {
    return _default;
  }

  if (typeof dict[key] === "undefined") {
    return _default;
  }

  return dict[key];
};

var parse_json_with_check = function parse_json_with_check(str) {
  if (typeof str === "undefined") {
    log_warning("Tried to parse an undefined JSON string");
    return false;
  }

  if (typeof str !== "string") {
    log_warning("Tried to parse a JSON string that is not a string: " + JSON.stringify(str));
    return false;
  }

  try {
    var result = JSON.parse(str);
    return result;
  } catch (e) {
    log_warning("ADCController encountered a JSON parsing error with string: " + str);
  }

  return false;
};

var replace_keys = function replace_keys(dict, mapping) {
  for (var key in dict) {
    var new_key = lookup_string(mapping, key);
    if (new_key !== "") {
      dict[new_key] = dict[key];
      delete dict[key];
    }
  }
  return dict;
};

var merge_dictionaries = function merge_dictionaries(dict1, dict2) {
  var new_dict = {};

  for (var key in dict1) {
    new_dict[key] = dict1[key];
  }

  for (var key in dict2) {
    new_dict[key] = dict2[key];
  }

  return new_dict;
};

function ADC_get_sentinel() {
  return 37;
}

var ADCMessageDispatcher = (function () {
  function ADCMessageDispatcher() {
    _classCallCheck(this, ADCMessageDispatcher);

    this.current_message_id = 0;
    this.message_handlers = {};
    this.message_reply_handlers = {};
    this.outbound = [];
    this.max_messages = 1024;
  }

  ADCMessageDispatcher.prototype.next_message_id = function next_message_id() {
    return this.current_message_id++;
  };

  ADCMessageDispatcher.prototype.queue_outbound_message = function queue_outbound_message(message, reply_handler) {
    if (message.m_id == undefined) {
      message.m_id = this.next_message_id();
    }

    var message_id = message.m_id;

    this.outbound.push(message);

    if (typeof reply_handler === "undefined" || reply_handler === null || typeof reply_handler !== "function") {
      return;
    }
    if (lookup(this.message_reply_handlers, message_id) === "undefined") {
      this.message_reply_handlers[message_id] = reply_handler;
    }

    return message_id;
  };

  ADCMessageDispatcher.prototype.register_message_handler = function register_message_handler(handler, message_type) {
    log_trace("Registering message handler for message type: " + message_type);

    if (lookup(this.message_handlers, message_type) === "undefined") {
      this.message_handlers[message_type] = [];
    }

    var handlers = lookup_array(this.message_handlers, message_type);
    for (var i = 0; i < handlers.length; ++i) {
      var first_handler = handlers[i];
      if (first_handler === handler) {
        log_warning("Tried to register multiple instances of the same message handler. Ignoring");
        return;
      }
    }

    this.message_handlers[message_type].push(handler);
  };

  ADCMessageDispatcher.prototype.unregister_reply_handler = function unregister_reply_handler(message_id) {
    log_trace("Unregistering message reply handler for message with id: " + message_id);
    delete this.message_reply_handlers[message_id];
  };

  ADCMessageDispatcher.prototype["package"] = function _package() {
    var how_many = this.outbound.length > this.max_messages ? this.max_messages : this.outbound.length;
    var outbound_messages = JSON.stringify(this.outbound.splice(0, how_many));

    return outbound_messages;
  };

  ADCMessageDispatcher.prototype.distribute = function distribute(inbound_messages) {
    inbound_messages = inbound_messages || [];

    for (var i = 0; i < inbound_messages.length; ++i) {
      var message = inbound_messages[i];
      var message_id = lookup_number(message, "m_id");
      var message_type = lookup_string(message, "m_type");

      if (message_type === "") {
        log_warning("Invalid message received. ignoring");
        continue;
      }

      if (message_id === -1) {
        log_warning("Invalid message received. ignoring");
        continue;
      }

      if (message_type === "reply") {
        if (lookup(this.message_reply_handlers, message_id) === "undefined") {
          log_info("No reply handler registered for message with id: " + message_id);
          continue;
        }

        var handler = this.message_reply_handlers[message_id];
        handler(message);
        this.unregister_reply_handler(message_id);
      } else {

        if (lookup_array(this.message_handlers, message_type).length === 0) {
          log_info("No handler has been registered to message type: " + message_type);
          continue;
        }

        var handlers = lookup_array(this.message_handlers, message_type);
        for (var j = 0; j < handlers.length; ++j) {
          var handler = handlers[j];
          handler(message);
        }
      }
    }
  };

  return ADCMessageDispatcher;
})();

var acni = (function (global) {
  var acni = {};
  acni.event_listeners = {};

  acni.init = function (module_id, os_name, pass_messages) {
    acni.module_id = module_id;
    acni.os_name = os_name;
    acni.pass_messages = pass_messages;
  };

  acni.add_event_listener = function (type, uuid, handler) {
    var listeners = acni.event_listeners[type];
    if (listeners) {
      listeners[uuid] = handler;
    } else {
      var obj = {};
      obj[uuid] = handler;
      acni.event_listeners[type] = obj;
    }
  };

  acni.remove_event_listener = function (type, uuid) {
    var listeners = acni.event_listeners[type];
    if (listeners) {
      var handler = listeners[uuid];
      if (handler) {
        delete listeners[uuid];
      }
    }
  };

  acni.on_event = function (type, uuid, data) {
    var listeners = acni.event_listeners[type];
    if (listeners) {
      var handler = listeners[uuid];
      if (handler) {
        handler(data);
      }
    }
  };

  acni.ad_unit_on_left_application = function (module_id, ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.left_application",
      "m_origin": module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id
    });
  };

  acni.ad_unit_on_click = function (module_id, ad_session_id) {
    log_info("acni.ad_unit_on_click called");

    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.click",
      "m_origin": module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id
    });
  };

  acni.get_native_app_info = function (reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler({ "options": AdColony__getAppInfo() });
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "AdColony.get_app_info",
        "m_origin": acni.module_id,
        "m_target": 0
      }, reply_handler);
    }
  };

  acni.v4vc_reward = function (success, reward_amount, reward_name, zone_id) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      AdColony__V4VCReward(success, reward_amount, reward_name, zone_id);
    } else {
      var message = {
        "m_type": "AdColony.v4vc_reward",
        "m_origin": acni.module_id,
        "m_target": 0,
        "success": success,
        "reward_amount": reward_amount,
        "reward_name": reward_name,
        "zone_id": zone_id
      };
      $.message_dispatcher.queue_outbound_message(message);
    }
  };

  acni.zone_info = function (zone_id, reward, type, _status, app_session_id, rewarded, play_interval) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      AdColony__zoneInfo(zone_id, reward, type, _status, app_session_id, rewarded, play_interval);
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "AdColony.zone_info",
        "m_origin": acni.module_id,
        "m_target": 0,
        "zone_id": zone_id,
        "reward": reward,
        "type": type,
        "status": _status,
        "app_session_id": app_session_id,
        "rewarded": rewarded,
        "play_interval": play_interval
      });
    }

    log_info("Zone_info with status = " + _status);
  };

  acni.on_configured = function (app_session_id) {
    if (acni.os_name !== "android" && !acni.pass_messages) {} else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "AdColony.on_configured",
        "m_origin": acni.module_id,
        "m_target": 0,
        "app_session_id": app_session_id,
        "concurrent_requests": 16 + 2
      });
    }
  };

  acni.on_configure_error = function () {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      AdColony__onConfigError();
    } else {}
  };

  acni.disable_adcolony = function () {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      AdColony__disable();
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "AdColony.disable",
        "m_origin": acni.module_id,
        "m_target": 0
      });
    }
  };

  acni.send_controller_version = function (version) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      AdColony__sendControllerVersion(version);
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "AdColony.controller_version",
        "m_origin": acni.module_id,
        "m_target": 0,
        "version": version
      });
    }
  };

  acni.start_fullscreen_ad = function (ad_session_id, orientation) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.start_fullscreen_ad",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id,
      "orientation": orientation,
      "module_id": acni.module_id
    });
  };

  acni.finish_fullscreen_ad = function (ad_session_id, _status, fullscreen, v4iap, reply_handler) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.finish_fullscreen_ad",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id,
      "status": _status,
      "container_id": fullscreen,
      "v4iap": v4iap
    }, reply_handler);
  };

  acni.interstitial_available = function (ad_session_id, iap_enabled, filepath, ad_id, creative_id, ias) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.interstitial_available",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id,
      "iap_enabled": iap_enabled,
      "video_filepath": filepath,
      "ad_id": ad_id,
      "creative_id": creative_id,
      "ias": ias
    });
  };

  acni.interstitial_unavailable = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.interstitial_unavailable",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id
    });
  };

  acni.native_ad_view_available = function (ad_session_id, iap_enabled) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.native_ad_view_available",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id,
      "iap_enabled": enabled
    });
  };

  acni.native_ad_view_unavailable = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.native_ad_view_unavailable",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id
    });
  };

  acni.destroy_ad_session = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.destroy",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id
    });
  };

  acni.audio_started = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.audio_started",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id
    });
  };

  acni.audio_stopped = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.audio_stopped",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": ad_session_id
    });
  };

  acni.native_engagement = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.native_engagement",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id
    });
  };

  acni.expiration = function (ad_session_id, type) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdSession.expiring",
      "m_origin": 1,
      "m_target": 0,
      "id": ad_session_id,
      "type": type
    });
  };

  acni.set_supported_orientations = function (ad_session_id, orientation) {
    var dict = {
      "id": ad_session_id,
      "orientation": orientation
    };

    dict["m_type"] = "AdSession.set_supported_orientations";
    dict["m_origin"] = acni.module_id;
    dict["m_target"] = 0;
    $.message_dispatcher.queue_outbound_message(dict);
  };

  acni.change_orientation = function (ad_session_id, orientation) {
    var dict = {
      "id": ad_session_id,
      "orientation": orientation
    };

    dict["m_type"] = "AdSession.change_orientation";
    dict["m_origin"] = acni.module_id;
    dict["m_target"] = 0;
    $.message_dispatcher.queue_outbound_message(dict);
  };

  acni.has_audio = function (ad_session_id, reply_handler) {
    var dict = {
      "id": ad_session_id
    };
    dict["m_type"] = "AdSession.has_audio";
    dict["m_origin"] = acni.module_id;
    dict["m_target"] = 0;
    $.message_dispatcher.queue_outbound_message(dict, reply_handler);
  };

  acni.on_ad_unit_event = function (ad_session_id, custom_data) {
    var message_dict = {
      "m_type": "AdUnit.on_event",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id
    };

    message_dict = merge_dicts(message_dict, custom_data);
    $.message_dispatcher.queue_outbound_message(message_dict);
  };

  acni.on_ad_unit_in_video_engagement = function (ad_session_id, custom_data) {
    var message_dict = {
      "m_type": "AdUnit.on_in_video_engagement",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id
    };

    message_dict = merge_dicts(message_dict, custom_data);
    $.message_dispatcher.queue_outbound_message(message_dict);
  };

  acni.ad_unit_on_expand = function (origin_module_id, target_module_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.expanded",
      "m_origin": origin_module_id,
      "m_target": target_module_id
    });
  };

  acni.ad_unit_on_start = function (ad_session_id, replay, reward, from_native) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_start",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward,
      "from_native": from_native
    });
  };

  acni.ad_unit_on_pause = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_pause",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id
    });
  };

  acni.ad_unit_on_resume = function (ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_resume",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id
    });
  };

  acni.ad_unit_on_replay = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_replay",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_unit_on_finish = function (zone_id, ad_session_id, _status, post_popup, replay, reward, dec_load_start, dec_load_finish, dec_load_time, dec_load_timeout, dec_time_spent, dec_dissolved, video_finished, skip_type, iap_engagement, reply_handler) {
    var message_dict = {
      "m_type": "AdUnit.on_finish",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "zone_id": zone_id,
      "status": _status,
      "post_popup": post_popup,
      "replay": replay,
      "reward": reward,
      "html_endcard_loading_started": dec_load_start,
      "html_endcard_loading_finished": dec_load_finish,
      "html_endcard_loading_time": dec_load_time,
      "html_endcard_load_timeout": dec_load_timeout,
      "endcard_time_spent": dec_time_spent,
      "endcard_dissolved": dec_dissolved,
      "skip_type": skip_type,
      "video_finished": video_finished,
      "iap_engagement": iap_engagement
    };

    $.message_dispatcher.queue_outbound_message(message_dict, reply_handler);
  };

  acni.ad_unit_on_video_first_quartile = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_video_first_quartile",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_unit_on_video_midpoint = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_video_midpoint",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_unit_on_video_third_quartile = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_video_third_quartile",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_unit_on_video_finish = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_video_finish",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_unit_on_card_shown = function (ad_session_id, replay, reward) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_card_shown",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward
    });
  };

  acni.ad_on_reward_v4vc = function (ad_session_id, replay, reward, custom_data) {
    var message_dict = {
      "m_type": "AdUnit.on_reward_v4vc",
      "m_origin": acni.module_id,
      "m_target": 1,
      "ad_session_id": ad_session_id,
      "replay": replay,
      "reward": reward,
      "custom_data": custom_data
    };

    $.message_dispatcher.queue_outbound_message(message_dict);
  };

  acni.ad_unit_on_v4vc_request = function (ad_session_id, zone_id, replay, reward_amount, reward_name) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AdUnit.on_v4vc_request",
      "m_origin": acni.module_id,
      "m_target": 1,
      "reward_amount": reward_amount,
      "reward_name": reward_name,
      "zone_id": zone_id,
      "replay": replay,
      "ad_session_id": ad_session_id
    });
  };

  acni.show_alert = function (ad_session_id, container_id, message, title, positive_action, negative_action, on_resume, reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      acni.add_event_listener("Alert.on_dismissed", ad_session_id, function (positive) {
        reply_handler({ "positive": positive ? true : false });
        acni.remove_event_listener("Alert.on_dismissed", ad_session_id);
      });

      Alert__show(ad_session_id, title, message, positive_action, negative_action);
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "Alert.show",
        "m_origin": acni.module_id,
        "m_target": 0,
        "ad_session_id": ad_session_id,
        "container_id": container_id,
        "message": message,
        "title": title,
        "positive": positive_action,
        "negative": negative_action,
        "on_resume": on_resume
      }, reply_handler);
    }
  };

  acni.create_audio_player = function (ad_session_id, player_id, filepath, repeats, use_sound_pool) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AudioPlayer.create",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "id": player_id,
      "filepath": filepath,
      "repeats": repeats,
      "use_sound_pool": use_sound_pool
    });
  };

  acni.play_audio_player = function (ad_session_id, player_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AudioPlayer.play",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "id": player_id
    });
  };

  acni.pause_audio_player = function (ad_session_id, player_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AudioPlayer.pause",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "id": player_id
    });
  };

  acni.stop_audio_player = function (ad_session_id, player_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AudioPlayer.stop",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "id": player_id
    });
  };

  acni.destroy_audio_player = function (ad_session_id, player_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "AudioPlayer.destroy",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "id": player_id
    });
  };

  acni.crc32 = function (data, reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler({ "uuids": Crypto__CRC32(data) });
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "Crypto.uuid",
        "m_origin": acni.module_id,
        "m_target": 0,
        "data": data
      }, reply_handler);
    }
  };

  acni.sha1 = function (data, reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler({ "uuids": Crypto__SHA1(data) });
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "Crypto.uuid",
        "m_origin": acni.module_id,
        "m_target": 0,
        "data": data
      }, reply_handler);
    }
  };

  acni.uuid = function (number, reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler({ "uuids": Crypto__UUID(number) });
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "Crypto.uuid",
        "m_origin": acni.module_id,
        "m_target": 0,
        "number": number
      }, reply_handler);
    }
  };

  acni.send_custom_message = function (type, message) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "CustomMessage.controller_send",
      "m_origin": acni.module_id,
      "m_target": 0,
      "type": type,
      "message": message
    });
  };

  acni.send_custom_message_reply = function (reply_id, obj) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "CustomMessage.controller_reply",
      "m_origin": acni.module_id,
      "m_target": 0,
      "id": reply_id,
      "obj": obj
    });
  };
  acni.get_device_info = function (reply_handler) {
    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(Device__getInfo());
    } else {
      $.message_dispatcher.queue_outbound_message({
        "m_type": "Device.get_info",
        "m_origin": acni.module_id,
        "m_target": 0
      }, reply_handler);
    }
  };

  acni.update_device_info = function (info) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "Device.get_info",
      "m_origin": acni.module_id,
      "m_target": 0,
      "info": info
    });
  };

  acni.query_advertiser_info = function (reply_handler) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "Device.query_advertiser_info",
      "m_origin": 1,
      "m_target": 0
    }, reply_handler);
  };

  acni.file_exists = function (filepath, reply_handler) {
    var dict = {
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__exists(dict));
    } else {
      dict["m_type"] = "FileSystem.exists";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };
  acni.load_file = function (filepath, decrypt, calculate_crc32, encoding, reply_handler) {
    var dict = {
      "filepath": filepath,
      "decrypt": decrypt,
      "calculate_crc32": calculate_crc32,
      "encoding": encoding || "utf8"
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__load(dict));
    } else {
      dict["m_type"] = "FileSystem.load";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.rename_file = function (filepath, new_filepath, reply_handler) {
    var dict = {
      "filepath": filepath,
      "new_filepath": new_filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__rename(dict));
    } else {
      dict["m_type"] = "FileSystem.rename";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.save_file = function (filepath, data, encrypt, encoding, reply_handler) {
    var dict = {
      "filepath": filepath,
      "data": data,
      "encrypt": encrypt,
      "encoding": encoding
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__save(dict));
    } else {
      dict["m_type"] = "FileSystem.save";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.delete_file = function (filepath, reply_handler) {
    var dict = {
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      if (typeof reply_handler === "undefined" || reply_handler === null || typeof reply_handler !== "function") {
        FileSystem__delete(dict);
      } else {
        reply_handler(FileSystem__delete(dict));
      }
    } else {
      dict["m_type"] = "FileSystem.delete";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };
  acni.extract_from_file = function (filepath, offset, size, output_filepath, gunzip, reply_handler) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "FileSystem.extract",
      "m_origin": acni.module_id,
      "m_target": 0,
      "filepath": filepath,
      "offset": offset,
      "size": size,
      "output_filepath": output_filepath,
      "gunzip": gunzip
    }, reply_handler);
  };
  acni.directory_listing = function (filepath, reply_handler) {
    var dict = {
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__listing(dict));
    } else {
      dict["m_type"] = "FileSystem.listing";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.delete_directory = function (filepath, reply_handler) {
    var dict = {
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      if (typeof reply_handler !== "undefined") {
        reply_handler(FileSystem__delete(dict));
      }
    } else {

      dict["m_type"] = "FileSystem.delete";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.create_directory = function (filepath, reply_handler) {
    var dict = {
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(FileSystem__createDirectory(dict));
    } else {
      dict["m_type"] = "FileSystem.create_directory";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.cache_media_file = function (url, reply_handler) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "MediaPool.cache",
      "m_origin": acni.module_id,
      "m_target": 0,
      "url": url
    }, reply_handler);
  };

  acni.load_module = function (ad_session_id, id, info, is_webview, use_uiwebview, is_display_module, data, filepath, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "id": id,
      "info": info,
      "is_webview": is_webview,
      "use_uiwebview": use_uiwebview,
      "is_display_module": is_display_module,
      "data": data,
      "filepath": filepath
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(Module__load(dict));
    } else {
      dict["m_type"] = "Module.load", dict["m_origin"] = acni.module_id, dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.unload_module = function (ad_session_id) {
    var dict = {
      "id": acni.module_id,
      "ad_session_id": ad_session_id
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Module__unload(dict);
    } else {
      dict["m_type"] = "Module.unload", dict["m_origin"] = acni.module_id, dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.mraid_on_replay = function (origin_module_id, target_module_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "MRAID.on_replay",
      "m_origin": origin_module_id,
      "m_target": target_module_id
    });
  };

  acni.mraid_on_viewable_change = function (origin_module_id, target_module_id, visible) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "MRAID.viewable_change",
      "m_origin": origin_module_id,
      "m_target": target_module_id,
      "visible": visible
    });
  };

  acni.prepare_browser = function (config) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.prepare_browser",
      "m_origin": acni.module_id,
      "m_target": 0,
      "config": config
    });
  };

  acni.open_browser = function (url, in_app, ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.open_browser",
      "m_origin": acni.module_id,
      "m_target": 0,
      "url": url,
      "in_app": in_app,
      "ad_session_id": ad_session_id
    });
  };

  acni.open_store = function (product_id, ad_session_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.open_store",
      "m_origin": acni.module_id,
      "m_target": 0,
      "product_id": product_id,
      "ad_session_id": ad_session_id
    });
  };

  acni.disable_system_service = function (service) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "System.disable_service",
      "m_origin": acni.module_id,
      "m_target": 0,
      "service": service
    });
  };

  acni.create_ad_container = function (ad_session_id, fullscreen, width, height, viewability, advanced_viewability, supported_orientations, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "id": fullscreen ? 1 : 0,
      "module_id": acni.module_id,
      "fullscreen_orientation": fullscreen,
      "width": width,
      "height": height,
      "viewability_enabled": viewability,
      "advanced_viewability": advanced_viewability,
      "supported_orientations": supported_orientations
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("AdContainer__create(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "AdContainer.create", dict["m_origin"] = acni.module_id, dict["m_target"] = 0, $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.destroy_ad_container = function (ad_session_id, fullscreen, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "id": fullscreen ? 1 : 0
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("AdContainer__destroy(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "AdContainer.destroy", dict["m_origin"] = acni.module_id, dict["m_target"] = 0, $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.move_view_to_front = function (ad_session_id, fullscreen, view_id, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "id": fullscreen,
      "view_id": view_id
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("AdContainer__moveViewToFront(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "AdContainer.move_view_to_front", dict["m_origin"] = acni.module_id, dict["m_target"] = 0, $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.set_web_view_scrolling = function (ad_session_id, enabled, container_id, view_id) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "WebView.set_scrollable",
      "m_origin": acni.module_id,
      "m_target": 0,
      ad_session_id: ad_session_id,
      container_id: container_id,
      id: view_id,
      enabled: !!enabled
    });
  };

  acni.move_view_to_index = function (ad_session_id, fullscreen, view_id, index, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "id": fullscreen,
      "view_id": view_id,
      "index": index
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("AdContainer__moveViewToIndex(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "AdContainer.move_view_to_index", dict["m_origin"] = acni.module_id, dict["m_target"] = 0, $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.create_view = function (type, ad_session_id, container_id, view_id, x, y, w, h, custom_data, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "x": x,
      "y": y,
      "width": w,
      "height": h
    };
    dict = merge_dicts(dict, custom_data);

    if (acni.os_name !== "android" && !acni.pass_messages && (type == "VideoView" || type == "ImageView")) {
      Function(type + "__create(" + JSON.stringify(dict) + ");")();
    } else {

      dict["m_type"] = type + ".create";
      if (!dict["m_origin"]) {
        dict["m_origin"] = acni.module_id;
      }
      dict["m_target"] = 0;

      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };

  acni.set_view_visibility = function (type, ad_session_id, container_id, view_id, visible, custom_data) {
    var dict = merge_dicts({
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "visible": visible,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    }, custom_data || {});

    if (acni.os_name !== "android" && !acni.pass_messages && (type == "Button" || type == "VideoView" || type == "ImageView")) {
      Function(type + "__setVisible(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = type + ".set_visible";
      if (!dict["m_origin"]) {
        dict["m_origin"] = acni.module_id;
      }
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.set_view_bounds = function (type, ad_session_id, container_id, view_id, x, y, w, h) {
    var dict = {
      "id": view_id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "x": x,
      "y": y,
      "width": w,
      "height": h
    };

    if (acni.os_name !== "android" && !acni.pass_messages && (type == "Button" || type == "VideoView" || type == "ImageView")) {
      Function(type + "__setBounds(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = type + ".set_bounds";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.destroy_view = function (type, ad_session_id, container_id, view_id) {
    var dict = {
      "id": view_id,
      "ad_session_id": ad_session_id,
      "container_id": container_id
    };

    if (acni.os_name !== "android" && !acni.pass_messages && (type == "Button" || type == "VideoView" || type == "ImageView")) {
      Function(type + "__destroy(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = type + ".destroy";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };
  acni.create_video_view = function (ad_session_id, container_id, view_id, x, y, w, h, custom_data) {
    acni.create_view("VideoView", ad_session_id, container_id, view_id, x, y, w, h, custom_data);
  };

  acni.play_video = function (view_id, ad_session_id, container_id, replay) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "replay": replay,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("VideoView__play(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "VideoView.play";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.pause_video = function (view_id, ad_session_id, container_id) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("VideoView__pause(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "VideoView.pause";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };
  acni.seek_video_to_time = function (view_id, ad_session_id, container_id, time, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "time": time,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(VideoView__seek(dict));
    } else {
      dict["m_type"] = "VideoView.seek_to_time";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };
  acni.set_video_view_volume = function (view_id, ad_session_id, container_id, volume, reply_handler) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": view_id,
      "volume": volume
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      reply_handler(VideoView__setVolume(dict));
    } else {
      dict["m_type"] = "VideoView.set_volume";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict, reply_handler);
    }
  };
  acni.create_web_view = function (ad_session_id, container_id, id, x, y, w, h, custom_data, reply_handler) {
    acni.create_view("WebView", ad_session_id, container_id, id, x, y, w, h, custom_data, reply_handler);
  };

  acni.execute_js = function (ad_session_id, container_id, web_view_id, custom_js) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "WebView.execute_js",
      "m_origin": acni.module_id,
      "m_target": 0,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": web_view_id,
      "custom_js": custom_js
    });
  };

  acni.create_image_view = function (ad_session_id, container_id, id, x, y, w, h, custom_data) {
    acni.create_view("ImageView", ad_session_id, container_id, id, x, y, w, h, custom_data);
  };

  acni.set_image_view_image = function (ad_session_id, container_id, image_view_id, filepath) {
    var dict = {
      "id": image_view_id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "filepath": filepath,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("ImageView__setImage(" + JSON.stringify(dict) + ");")();
    } else {
      dict["m_type"] = "ImageView.set_image", dict["m_origin"] = acni.module_id, dict["m_target"] = 0, $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.create_text_view = function (ad_session_id, container_id, id, x, y, w, h, custom_data) {
    acni.create_view("TextView", ad_session_id, container_id, id, x, y, w, h, custom_data);
  };

  acni.set_text_view_text = function (ad_session_id, container_id, id, text) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "text": text,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("TextView__setText(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.get_text_view_text = function (ad_session_id, container_id, id) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__getText(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.set_text_view_typeface = function (ad_session_id, container_id, id, typeface) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "typeface": typeface,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__setTypeface(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.set_text_view_font_size = function (ad_session_id, container_id, id, size) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "size": size,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__setFontSize(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.set_text_view_font_color = function (ad_session_id, container_id, id, color) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "color": color,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__setFontColor(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.set_text_view_background_color = function (ad_session_id, container_id, id, color) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "color": color,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__setBackgroundColor(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.set_text_view_editable = function (ad_session_id, container_id, id, editable) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "editable": editable,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      return Function("TextView__setEditable(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.create_color_view = function (ad_session_id, container_id, id, x, y, w, h, custom_data) {
    acni.create_view("ColorView", ad_session_id, container_id, x, y, w, h, custom_data);
  };

  acni.set_color_view_color = function (ad_session_id, container_id, id, color_hex_string) {
    var dict = {
      "id": id,
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "color": color_hex_string,
      "x": 1,
      "y": 1,
      "width": 1,
      "height": 1
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Function("ColorView__setColor(" + JSON.stringify(dict) + ");")();
    } else {}
  };

  acni.create_button = function (ad_session_id, container_id, id, x, y, w, h, normal_filepath, down_filepath, web_content, text, html, web_view_id) {
    var dict = {
      "ad_session_id": ad_session_id,
      "container_id": container_id,
      "id": id,
      "x": x,
      "y": y,
      "width": w,
      "height": h,
      "normal_image_filepath": normal_filepath,
      "down_image_filepath": down_filepath,
      "web_content_enabled": web_content,
      "text": text,
      "html": html,
      "web_view_id": web_view_id
    };

    if (acni.os_name !== "android" && !acni.pass_messages) {
      Button__create(dict);
    } else {
      dict["m_type"] = "Button.create";
      dict["m_origin"] = acni.module_id;
      dict["m_target"] = 0;
      $.message_dispatcher.queue_outbound_message(dict);
    }
  };

  acni.add_network_request = function (type, url, is_third_party, is_v4vc, no_redirect, no_cellular, payload, max_size, completion) {
    var request_type = "get";
    var message_type = "WebServices.get";
    if (typeof payload !== "undefined" && payload !== null) {
      request_type = "post";
      message_type = "WebServices.post";
      var content_type = "application/json";
    }

    if (payload === null) {
      payload = undefined;
    }

    if (!max_size) {
      max_size = 0;
    }

    acni.uuid(1, function (reply) {
      var uuids = lookup_array(reply, "uuids");
      if (uuids.length === 0) {
        return;
      }

      $.message_dispatcher.queue_outbound_message({
        "m_type": "Controller.queue_network_request",
        "m_origin": acni.module_id,
        "m_target": 1,
        "uuid": uuids[0],
        "request_config": {
          "m_id": $.message_dispatcher.next_message_id(),
          "m_type": message_type,
          "m_origin": acni.module_id,
          "m_target": 0,
          "request_type": request_type,
          "type": type,
          "url": url,
          "max_size": max_size,
          "content_type": content_type,
          "no_redirect": no_redirect,
          "no_cellular": no_cellular,
          "is_third_party": is_third_party,
          "is_v4vc": is_v4vc,
          "content": payload,
          "uuid": uuids[0]
        }
      });

      if (typeof completion == "function") {
        acni.add_event_listener("NetworkRequest.on_processed", uuids[0], function () {
          acni.remove_event_listener("NetworkRequest.on_processed", uuids[0]);
          completion.apply(undefined, arguments);
        });
      }
    });
  };

  acni.start_network_request = function (request_config, completion) {
    var uuid = lookup_string(request_config, "uuid");
    var request_type = lookup_string(request_config, "request_type");
    acni.add_event_listener(request_type + ".on_complete", uuid, function (results) {
      completion(results);
      acni.remove_event_listener(request_type + ".on_complete", uuid);
    });

    Function("WebServices__" + request_type + "(" + JSON.stringify(request_config) + ");")();
  };

  acni.start_download = function (filename, download_config, completion) {
    acni.add_event_listener("Download.on_complete", filename, function (results) {
      completion(results);
      acni.remove_event_listener("Download.on_complete", filename);
    });

    WebServices__download(download_config);
  };

  acni.prepare_dec = function (mraid_filepath, dec_filepath, metadata, reply_handler) {
    $.message_dispatcher.queue_outbound_message({
      "m_type": "WebView.prepare",
      "m_origin": acni.module_id,
      "m_target": 0,
      "mraid": mraid_filepath,
      "filepath": dec_filepath,
      "metadata": metadata
    }, reply_handler);
  };

  var merge_dicts = function merge_dicts(dict1, dict2) {
    var new_dict = {};

    for (var key in dict1) {
      new_dict[key] = dict1[key];
    }

    for (var key in dict2) {
      new_dict[key] = dict2[key];
    }

    return new_dict;
  };

  return acni;
})();

var ADCAdUnit = (function () {
  function ADCAdUnit(module_id, ad_unit_config) {
    _classCallCheck(this, ADCAdUnit);

    this.module_id = module_id;
    this.id = lookup_number(ad_unit_config, "id");
    this.ad_session_id = lookup_string(ad_unit_config, "ad_session_id");
    this.zone_id = lookup_string(ad_unit_config, "zone_id");
    this.os_name = lookup_string(ad_unit_config, "os_name");
    this.from_native = lookup_boolean(lookup(ad_unit_config, "message"), "from_native");
    this.last_resume = 0;

    this.showing_alert = false;
    this.next_view_id = 0;
    this.expanded = lookup_number(ad_unit_config, "expanded");
    this.screen_width = lookup_number(ad_unit_config, "display_width");
    this.screen_height = lookup_number(ad_unit_config, "display_height");

    this.orientations = lookup_number(ad_unit_config, "supported_orientations");
    this.orientation = lookup_number(ad_unit_config, "orientation");

    this.safe_inset_top = 0;
    this.safe_inset_left = 0;
    this.safe_inset_bottom = 0;
    this.safe_inset_right = 0;

    this.video_path = lookup_string(ad_unit_config, "video_path");
    this.video_width = lookup_number(ad_unit_config, "video_width");
    this.video_height = lookup_number(ad_unit_config, "video_height");
    this.video_duration = lookup_number(ad_unit_config, "video_duration");
    this.skippable = lookup_boolean(ad_unit_config, "skippable");
    this.skip_delay = lookup_number(ad_unit_config, "skip_delay");
    this.skip_type = lookup_string(ad_unit_config, "skip_type");
    this.skip_delay_met = false;
    this.haptic_filepath = lookup_string(ad_unit_config, "haptic_filepath");
    this.elapsed = lookup_default(ad_unit_config, "elapsed", -1);
    this.button_paths = lookup(ad_unit_config, "button_paths");
    this.post_popup = lookup_boolean(ad_unit_config, "post_popup");

    this.first_quartile_sent = lookup_boolean(ad_unit_config, "first_quartile");
    this.midpoint_sent = lookup_boolean(ad_unit_config, "midpoint");
    this.third_quartile_sent = lookup_boolean(ad_unit_config, "third_quartile");
    this._start = lookup_boolean(ad_unit_config, "start");
    this.video_finished = false;
    this.replay = lookup_boolean(ad_unit_config, "replay");

    this.v4iap = lookup_default(ad_unit_config, "v4iap", {});
    this.v4iap_enabled = Object.keys(this.v4iap).length > 0 ? true : false;

    this.html_tag = lookup_string(ad_unit_config, "html_tag");
    this.mraid_filepath = lookup_string(ad_unit_config, "mraid_filepath");
    this.metadata = lookup(ad_unit_config, "metadata");
    this.dec_source_path = lookup_string(ad_unit_config, "dec_source_path");
    this.dec_source = "";
    this.dec_loaded = false;
    this.dec_load_start = false;
    this.dec_load_time = 0;
    this.dec_load_timeout = false;
    this.dec_time_spent = 0;
    this.dissolve = lookup_boolean(ad_unit_config, "dissolve");
    this.dissolve_delay = lookup_number(ad_unit_config, "dissolve_delay");
    this.dissolve_time = 0;
    this.dissolved = false;

    this.image_overlay = lookup_boolean(ad_unit_config, "image_overlay") && this.os_name === "android";
    this.text_overlay = lookup_boolean(ad_unit_config, "text_overlay") && this.os_name === "android";
    this.web_overlay = lookup_boolean(ad_unit_config, "web_overlay") && this.os_name === "ios";
    this.engagement_type = lookup_string(ad_unit_config, "engagement_type");
    this.engagement_action = lookup_string(ad_unit_config, "engagement_action");
    this.engagement_width = lookup_number(ad_unit_config, "engagement_width");
    this.engagement_height = lookup_number(ad_unit_config, "engagement_height");
    this.engagement_delay = lookup_number(ad_unit_config, "engagement_delay");
    this.engagement_event = lookup_string(ad_unit_config, "engagement_event");
    this.engagement_text = lookup_string(ad_unit_config, "label");
    this.engagement_label_html = lookup_string(ad_unit_config, "label_html");
    this.engagement_delay_met = false;
    this.image_normal_filepath = lookup_string(ad_unit_config, "image_normal_filepath");
    this.image_down_filepath = lookup_string(ad_unit_config, "image_down_filepath");

    this.viewability_enabled = lookup_boolean(ad_unit_config, "viewability_enabled");
    this.viewable_duration = lookup_number(ad_unit_config, "viewable_duration");
    this.viewable_percentage = lookup_number(ad_unit_config, "viewable_percentage");
    this.current_viewable_percentage = 0;
    this.current_viewable_percentage_time = 0;
    this.last_viewability_update = Date.now();
    this.on_viewable_impression_called = lookup_boolean(ad_unit_config, "viewable_impression");

    this.self_hosted_video_playing = false;
    this.skipped_to_dec = false;

    this.v4vc_enabled = lookup_boolean(ad_unit_config, "v4vc_enabled");
    this.client_side = lookup_boolean(ad_unit_config, "client_side");
    this.reward_name = lookup_string(ad_unit_config, "reward_name");
    this.reward_amount = lookup_number(ad_unit_config, "reward_amount");
    this.verified_reward = false;
    this.views_per_reward = lookup_number(ad_unit_config, "views_per_reward");
    this.v4vc_videos_watched = lookup_number(ad_unit_config, "v4vc_videos_watched");

    this.replay_wait_time = Date.now();

    this.mute_enabled = lookup_boolean(ad_unit_config, "mute_enabled");
    log_trace("Mute enabled: " + this.mute_enabled);
    this.mute_filepath = lookup_string(ad_unit_config, "mute_filepath");
    log_trace("Mute filepath: " + this.mute_filepath);
    this.unmute_filepath = lookup_string(ad_unit_config, "unmute_filepath");
    log_trace("Unmute filepath: " + this.unmute_filepath);
    this.mute_delay = lookup_number(ad_unit_config, "mute_delay");
    log_trace("Mute delay: " + this.mute_delay);
    this.mute_position = lookup_string(ad_unit_config, "mute_position");
    log_trace("Mute position: " + this.mute_position);
    this.auto_mute = lookup_boolean(ad_unit_config, "auto_mute");
    log_trace("Auto mute: " + this.auto_mute);
    this.mute_delay_met = false;
    this.unmute_delay_met = false;
    if (this.auto_mute) {
      this.muted = true;
    } else {
      this.muted = false;
    }

    log_trace("Ad unit module id: " + this.module_id);
    log_trace("Ad unit ad session id: " + this.ad_session_id);
    log_trace("Ad unit id: " + this.id);
    log_trace("Ad unit screen width: " + this.screen_width);
    log_trace("Ad unit screen height: " + this.screen_height);
    log_trace("Ad unit supported orientations: " + this.orientations);
    log_trace("Ad unit video path: " + this.video_path);
    log_trace("Ad unit HTML tag: " + this.html_tag);
    log_trace("Ad unit DEC source path: " + this.dec_source_path);
    log_trace("Ad unit views_per_reward:" + this.views_per_reward);
    log_trace("Ad unit videos_watched:" + this.v4vc_videos_watched);
  }

  ADCAdUnit.prototype.start = function start() {
    var _this = this;

    acni.init(this.module_id, this.os_name, this.os_name === "ios" ? false : true);

    acni.create_ad_container(this.ad_session_id, true, this.width, this.height, true, false, this.orientation, function (reply) {
      log_trace("AdContainer.create message successfully received reply");
    });

    acni.start_fullscreen_ad(this.ad_session_id, this.orientation);

    $.message_dispatcher.register_message_handler(function (message) {
      _this.finish_ad(true);
    }, "AdSession.on_error");

    $.message_dispatcher.register_message_handler(function (message) {
      _this.on_exposure_change(lookup_number(message, "exposure"));
    }, "AdContainer.on_exposure_change");

    $.message_dispatcher.register_message_handler(function (message) {
      _this.screen_width = lookup_default(message, "screen_width", _this.screen_width);
      _this.screen_height = lookup_default(message, "screen_height", _this.screen_height);
      _this.on_fullscreen_ad_started();
    }, "AdSession.on_fullscreen_ad_started");

    $.message_dispatcher.register_message_handler(function (message) {
      log_info("Ad unit detecting SessionInfo.on_pause message");

      acni.pause_video(_this.video_view_id, _this.ad_session_id, 1);

      if (!_this.muted) {
        acni.audio_stopped(_this.ad_session_id);
      }

      acni.ad_unit_on_pause(_this.ad_session_id);
    }, "SessionInfo.on_stop");

    $.message_dispatcher.register_message_handler(function (message) {
      log_info("Ad unit detecting SessionInfo.on_resume message");
      if (_this.showing_alert) {
        return;
      }

      acni.play_video(_this.video_view_id, _this.ad_session_id, 1);

      if (!_this.muted) {
        acni.audio_started(_this.ad_session_id);
      }

      acni.ad_unit_on_resume(_this.ad_session_id);
    }, "SessionInfo.on_start");

    $.message_dispatcher.register_message_handler(function (message) {

      _this.on_exposure_change(0);

      if (!_this.muted && _this.first_pause) {
        acni.audio_stopped(_this.ad_session_id);
      }
      _this.first_pause = true;
    }, "SessionInfo.on_pause");

    $.message_dispatcher.register_message_handler(function (message) {

      var window_focus = lookup_boolean(message, "from_window_focus");
      var limit = false;
      if (!window_focus) {
        _this.last_resume = new Date().getTime();
      } else if (new Date().getTime() - _this.last_resume < 1000) {
        limit = true;
      }

      if (!_this.muted && !limit) {
        acni.audio_started(_this.ad_session_id);
      }
    }, "SessionInfo.on_resume");

    $.message_dispatcher.register_message_handler(function (message) {
      acni.audio_stopped(_this.ad_session_id);

      if (_this.dec_loaded && _this.dec_time_spent !== 0) {
        _this.dec_time_spent = (Date.now() - _this.dec_time_spent) / 1000;
        log_trace("DEC time spent: " + _this.dec_time_spent);
      }

      if (!_this.video_finished && !_this.skipped_to_dec) {
        _this.dec_time_spent = 0;
      }

      var _status = _this.video_finished || _this.replay ? 0 : 5;
      var post_popup = !_this.skipped_to_dec && (_this.video_finished || _this.replay) || lookup(_this.v4iap, "engagement_type") !== 'undefined' ? _this.post_popup : false;
      acni.finish_fullscreen_ad(_this.ad_session_id, _status, 1);
      acni.ad_unit_on_finish(_this.zone_id, _this.ad_session_id, _status, post_popup, _this.replay, _this.v4vc_enabled, _this.dec_load_start, _this.dec_loaded, _this.dec_load_time, _this.dec_load_timeout, _this.dec_time_spent, _this.dissolved, _this.video_finished && !_this.skipped_to_dec, _this.skip_type, false);
    }, "AdSession.on_request_close");

    $.message_dispatcher.register_message_handler(function (message) {
      _this.self_hosted_video_view_id = _this.next_view_id++;
      _this.color_view_id = _this.next_view_id++;

      var custom_data = {
        "filepath": lookup_string(message, "url"),
        "enable_progress": true,
        "enable_timer": false
      };

      var custom_color_data = {
        "color": "ff000000"
      };

      acni.create_view("ColorView", _this.ad_session_id, 1, _this.color_view_id, 0, 0, _this.screen_width, _this.screen_height, custom_color_data);
      acni.set_view_visibility("ColorView", _this.ad_session_id, 1, _this.color_view_id, true);
      acni.create_video_view(_this.ad_session_id, 1, _this.self_hosted_video_view_id, 0, 0, _this.screen_width, _this.screen_height, custom_data);
      acni.set_view_visibility("VideoView", _this.ad_session_id, 1, _this.self_hosted_video_view_id, true);
      _this.self_hosted_video_playing = true;
    }, "AdUnit.self_hosted_video");

    $.message_dispatcher.register_message_handler(function (message) {
      log_info("Detecting user has left application in ad session with id: " + _this.ad_session_id);
      acni.ad_unit_on_left_application(_this.module_id, _this.ad_session_id);
    }, "AdSession.on_left_application");

    $.message_dispatcher.register_message_handler(function (message) {
      log_info("Detecting click event in ad session with id: " + _this.ad_session_id);
      acni.ad_unit_on_click(_this.module_id, _this.ad_session_id);
    }, "AdSession.on_click");
  };

  ADCAdUnit.prototype.on_fullscreen_ad_started = function on_fullscreen_ad_started() {
    var _this2 = this;

    this.video_view_id = this.next_view_id++;

    if (this.os_name === "android") {
      var w_ratio = this.screen_width / this.video_width;
      var h_ratio = this.screen_height / this.video_height;
      var factor = w_ratio > h_ratio ? h_ratio : w_ratio;
      var x_offset = (this.screen_width - this.video_width * factor) / 2;
      var y_offset = (this.screen_height - this.video_height * factor) / 2;

      var width = this.video_width * factor;
      var height = this.video_height * factor;
      var custom_data = {
        "video_width": this.video_width,
        "video_height": this.video_height,
        "enable_timer": true,
        "filepath": this.video_path,
        "haptic_filepath": this.haptic_filepath
      };
      acni.create_video_view(this.ad_session_id, 1, this.video_view_id, x_offset, y_offset, width, height, custom_data);
    } else {
      var custom_data = {
        "enable_timer": true,
        "filepath": this.video_path
      };
      acni.create_video_view(this.ad_session_id, 1, this.video_view_id, 0, 0, this.screen_width, this.screen_height, custom_data);
    }

    this.button_size = this.screen_height > this.screen_width ? this.screen_width / 10 : this.screen_height / 10;
    if (this.mute_enabled) {

      var mute_x = 0;
      var mute_y = 0;
      var final_mute_pos = this.get_safe_button_position("top_left");

      if (this.mute_position === "top_right" && !this.skippable) {
        mute_x = this.screen_width - this.button_size;
        final_mute_pos = this.get_safe_button_position("top_right");
      } else if (this.mute_position === "bottom_right" && !this.web_overlay && !this.image_overlay && !this.text_overlay) {
        mute_x = this.screen_width - this.button_size;
        mute_y = this.screen_height - this.button_size;
        final_mute_pos = this.get_safe_button_position("bottom_right");
      }

      var autoresizing;
      if (acni.os_name === "ios") {
        if (mute_x === 0 && mute_y === 0) {
          autoresizing = 1 << 2 | 1 << 5;
        } else if (mute_x !== 0 && mute_y === 0) {
          autoresizing = 1 << 0 | 1 << 5;
        } else if (mute_x === 0 && mute_y !== 0) {

          autoresizing = 1 << 2 | 1 << 3;
        } else if (mute_x !== 0 && mute_y !== 0) {
          autoresizing = 1 << 0 | 1 << 3;
        }
      }

      log_info("Creating image views for mute buttons");

      this.mute_id = this.next_view_id++;
      var image_data = {
        "filepath": this.mute_filepath,
        "autoresizing": autoresizing
      };
      acni.create_image_view(this.ad_session_id, 1, this.mute_id, final_mute_pos.x, final_mute_pos.y, this.button_size, this.button_size, image_data);

      this.unmute_id = this.next_view_id++;
      image_data = {
        "filepath": this.unmute_filepath,
        "autoresizing": autoresizing
      };
      acni.create_image_view(this.ad_session_id, 1, this.unmute_id, final_mute_pos.x, final_mute_pos.y, this.button_size, this.button_size, image_data);
    }

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_video_view_ready(message);
    }, "VideoView.on_ready");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_video_view_progress(message);
    }, "VideoView.on_progress");

    $.message_dispatcher.register_message_handler(function (message) {
      if (_this2.video_finished || _this2.skippable && _this2.skip_delay_met) {
        if (!_this2.self_hosted_video_playing) {
          _this2.on_close_pressed();
        } else {
          acni.destroy_view("VideoView", _this2.ad_session_id, 1, _this2.self_hosted_video_view_id);
          acni.destroy_view("ColorView", _this2.ad_session_id, 1, _this2.color_view_id);
          _this2.self_hosted_video_playing = false;
        }
      }
    }, "AdSession.on_back_button");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_container_resized(message);
    }, "AdContainer.on_orientation_change");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_container_touch_began(message);
    }, "AdContainer.on_touch_began");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_container_touch_ended(message);
    }, "AdContainer.on_touch_ended");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.on_container_touch_cancelled(message);
    }, "AdContainer.on_touch_cancelled");

    $.message_dispatcher.register_message_handler(function (message) {
      acni.play_video(_this2.video_view_id, _this2.ad_session_id, 1);
    }, "System.on_browser_closed");

    $.message_dispatcher.register_message_handler(function (message) {
      _this2.v4iap["engagement_type"] = 0;
      _this2.finish_ad(false);
    }, "AdUnit.make_in_app_purchase");
  };

  ADCAdUnit.prototype.on_container_resized = function on_container_resized(dict) {
    this.screen_width = lookup_number(dict, "screen_width");
    this.screen_height = lookup_number(dict, "screen_height");
    this.safe_inset_top = lookup_default(dict, "safe_inset_top", this.safe_inset_top);
    this.safe_inset_left = lookup_default(dict, "safe_inset_left", this.safe_inset_left);
    this.safe_inset_bottom = lookup_default(dict, "safe_inset_bottom", this.safe_inset_bottom);
    this.safe_inset_right = lookup_default(dict, "safe_inset_right", this.safe_inset_right);

    if (this.os_name === "android") {

      var w_ratio = this.screen_width / this.video_width;
      var h_ratio = this.screen_height / this.video_height;
      var factor = w_ratio > h_ratio ? h_ratio : w_ratio;
      var x_offset = (this.screen_width - this.video_width * factor) / 2;
      var y_offset = (this.screen_height - this.video_height * factor) / 2;

      this.button_size = this.screen_height > this.screen_width ? this.screen_width / 10 : this.screen_height / 10;

      acni.set_view_bounds("ImageView", this.ad_session_id, 1, this.close_button_id, this.screen_width - this.button_size, 0, this.button_size, this.button_size);

      acni.set_view_bounds("ImageView", this.ad_session_id, 1, this.replay_button_id, 0, 0, this.button_size, this.button_size);

      acni.set_view_bounds("VideoView", this.ad_session_id, 1, this.video_view_id, x_offset, y_offset, this.video_width * factor, this.video_height * factor);

      acni.set_view_bounds("WebView", this.ad_session_id, 1, this.web_view_id, 0, 0, this.screen_width, this.screen_height);

      if (this.image_overlay) {
        acni.set_view_bounds("ImageView", this.ad_session_id, 1, this.image_overlay_id, this.screen_width, this.screen_height, this.engagement_width, this.engagement_height);
      }
    } else if (this.os_name === "ios") {
      this.button_size = this.screen_height > this.screen_width ? this.screen_width / 10 : this.screen_height / 10;

      var safe_button_pos = this.get_safe_button_position("top_left");
      acni.set_view_bounds("ImageView", this.ad_session_id, 1, this.replay_button_id, safe_button_pos.x, safe_button_pos.y, this.button_size, this.button_size);

      safe_button_pos = this.get_safe_button_position("top_right");
      acni.set_view_bounds("ImageView", this.ad_session_id, 1, this.close_button_id, safe_button_pos.x, safe_button_pos.y, this.button_size, this.button_size);

      safe_button_pos = this.get_safe_button_position("top_left");
      if (this.mute_position === "top_right" && !this.skippable) {
        safe_button_pos = this.get_safe_button_position("top_right");
      } else if (this.mute_position === "bottom_right" && !this.web_overlay && !this.image_overlay && !this.text_overlay) {
        safe_button_pos = this.get_safe_button_position("bottom_right");
      }

      var safe_wv = this.get_safe_webview();
      acni.set_view_bounds("WebView", this.ad_session_id, 1, this.web_view_id, safe_wv.x, safe_wv.y, safe_wv.w, safe_wv.h);

      acni.set_view_bounds("VideoView", this.ad_session_id, 1, this.video_view_id, 0, 0, this.screen_width, this.screen_height);
    }
  };

  ADCAdUnit.prototype.on_container_touch_began = function on_container_touch_began(dict) {
    switch (lookup_number(dict, "view_id")) {
      case this.close_button_id:

        acni.set_image_view_image(this.ad_session_id, 1, this.close_button_id, this.button_paths["close_down"]);
        break;
      case this.replay_button_id:

        acni.set_image_view_image(this.ad_session_id, 1, this.replay_button_id, this.button_paths["replay_down"]);
        break;
    }
  };

  ADCAdUnit.prototype.on_container_touch_moved = function on_container_touch_moved(dict) {};

  ADCAdUnit.prototype.on_container_touch_ended = function on_container_touch_ended(dict) {
    var _this3 = this;

    switch (lookup_number(dict, "view_id")) {
      case this.text_overlay_id:
      case this.image_overlay_id:

        var custom_data = {
          "replay": this.replay,
          "reward": this.v4vc_enabled,
          "event_type": this.engagement_event
        };
        acni.on_ad_unit_in_video_engagement(this.ad_session_id, custom_data);

        if (this.v4iap_enabled) {

          this.skip_type = "video";
          this.v4iap["engagement_type"] = 1;
          this.finish_ad(false);
          return;
        }

        if (this.engagement_type === "url") {

          acni.open_browser(this.engagement_action, true, this.ad_session_id);
          acni.pause_video(this.video_view_id, this.ad_session_id, 1);
        } else if (this.engagement_type === "open_store" || this.engagement_type === "install") {

          acni.open_store(this.engagement_action, this.ad_session_id);
        }

        if (this.from_native) {
          acni.native_engagement(this.ad_session_id);
        }

        break;

      case this.close_button_id:

        acni.set_image_view_image(this.ad_session_id, 1, this.close_button_id, this.button_paths["close_normal"]);

        var view_x = lookup_number(dict, "view_x");
        var view_y = lookup_number(dict, "view_y");
        if (view_x > 0 && view_y < this.button_size) {
          this.on_close_pressed();
        }

        break;

      case this.replay_button_id:

        acni.set_image_view_image(this.ad_session_id, 1, this.replay_button_id, this.button_paths["replay_normal"]);

        if (Date.now() < this.replay_wait_time && this.os_name !== "android") {
          return;
        }

        var view_x = lookup_number(dict, "view_x");
        var view_y = lookup_number(dict, "view_y");
        if (view_x < this.button_size && view_y < this.button_size) {
          this.on_replay_pressed();
        }

        break;

      case this.mute_id:
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.mute_id, false);
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, true);
        acni.set_video_view_volume(this.video_view_id, this.ad_session_id, 1, 0, function (reply) {
          log_info("Video view volume muted");

          if (!_this3.muted) {
            var custom_data = {
              "replay": _this3.replay,
              "reward": _this3.v4vc_enabled,
              "event_type": "sound_mute"
            };
            acni.on_ad_unit_event(_this3.ad_session_id, custom_data);

            _this3.muted = true;

            acni.audio_stopped(_this3.ad_session_id);
          }
        });

        break;

      case this.unmute_id:
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, false);
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.mute_id, true);
        acni.set_video_view_volume(this.video_view_id, this.ad_session_id, 1, 1.0, function (reply) {
          log_info("Video view volume unmuted");

          if (_this3.muted) {
            var custom_data = {
              "replay": _this3.replay,
              "reward": _this3.v4vc_enabled,
              "event_type": "sound_unmute"
            };
            acni.on_ad_unit_event(_this3.ad_session_id, custom_data);

            _this3.muted = false;

            acni.audio_started(_this3.ad_session_id);
          }
        });

        break;
    }
  };

  ADCAdUnit.prototype.on_container_touch_cancelled = function on_container_touch_cancelled(dict) {
    switch (lookup_number(dict, "view_id")) {
      case this.close_button_id:
        acni.set_image_view_image(this.ad_session_id, 1, this.close_button_id, this.button_paths["close_normal"]);
        break;
      case this.replay_button_id:
        acni.set_image_view_image(this.ad_session_id, 1, this.replay_button_id, this.button_paths["replay_normal"]);
        break;
    }
  };

  ADCAdUnit.prototype.on_card_shown = function on_card_shown(message) {
    log_trace("Card shown");

    acni.ad_unit_on_card_shown(this.ad_session_id, this.replay, this.v4vc_enabled);
  };

  ADCAdUnit.prototype.on_store_kit_opened = function on_store_kit_opened() {
    acni.pause_video(this.video_view_id, this.ad_session_id, 1);
    if (!this.video_finished && !this.muted) {
      acni.audio_stopped(this.ad_session_id);
    }
    acni.ad_unit_on_pause(this.ad_session_id);
  };

  ADCAdUnit.prototype.on_store_kit_closed = function on_store_kit_closed() {
    acni.play_video(this.video_view_id, this.ad_session_id, 1);
    if (!this.video_finished && !this.muted) {
      acni.audio_started(this.ad_session_id);
    }
    acni.ad_unit_on_resume(this.ad_session_id);
  };

  ADCAdUnit.prototype.on_button_pressed = function on_button_pressed(button_id) {
    log_trace("Button press detected in view with ID: " + button_id);
    if (button_id !== this.overlay_button_id) {
      return;
    }

    acni.ad_unit_on_click(this.module_id, this.ad_session_id);

    if (this.engagement_event === "in_video_engagement") {
      var custom_data = {
        "replay": this.replay,
        "reward": this.v4vc_enabled,
        "event_type": this.engagement_event
      };
      acni.on_ad_unit_event(this.ad_session_id, custom_data);
    }

    if (this.v4iap_enabled) {
      this.v4iap["engagement_type"] = 1;
      this.video_finished = true;
      this.finish_ad(false);
    } else if (this.engagement_type === "url") {
      acni.open_browser(this.engagement_action, true);
      acni.pause_video(this.video_view_id, this.ad_session_id, 1);
    } else if (this.engagement_type === "open_store" || this.engagement_type === "install") {
      acni.open_store(this.engagement_action);
    }
  };

  ADCAdUnit.prototype.skip_to_dec = function skip_to_dec() {
    if (!this.dec_loaded) {

      this.skip_type = "video";
    }

    if (!this.replay) {
      this.skipped_to_dec = true;
      var custom_data = {
        "event_type": "skip",
        "replay": this.replay,
        "reward": this.v4vc_enabled,
        "skip_type": this.skip_type
      };
    }

    if (this.dec_loaded) {
      acni.on_ad_unit_event(this.ad_session_id, custom_data);
    }

    this.on_video_finished({});
    acni.pause_video(this.video_view_id, this.ad_session_id, 1);
    acni.seek_video_to_time(this.video_view_id, this.ad_session_id, 1, this.video_duration, function (reply) {});
  };

  ADCAdUnit.prototype.on_close_pressed = function on_close_pressed() {
    var _this4 = this;

    if (!this.v4vc_enabled || this.video_finished || this.replay) {
      if (this.video_finished || this.skip_type === "video") {
        this.finish_ad(false);
        return;
      } else {
        this.skip_to_dec();
      }
    }

    if (this.v4vc_enabled && !this.replay) {
      var message = "";
      if (this.v4iap_enabled) {
        message = "Completion or interacting with an in-app purchase engagement is required to receive your reward. Are you sure you want to skip?";
      } else {
        message = "Completion is required to receive your reward. Are you sure you want to skip?";
      }

      acni.show_alert(this.ad_session_id, 1, message, "V4VC", "Yes", "No", undefined, function (message_reply) {
        if (lookup_boolean(message_reply, "positive")) {
          if (_this4.skip_type === "video") {
            _this4.finish_ad(false);
            return;
          } else {
            _this4.skip_to_dec();
          }
        }
        acni.play_video(_this4.video_view_id, _this4.ad_session_id, 1);
        acni.ad_unit_on_resume(_this4.ad_session_id);
        _this4.showing_alert = false;

        _this4.on_exposure_change(100);
      });
      this.showing_alert = true;
      acni.pause_video(this.video_view_id, this.ad_session_id, 1);

      this.on_exposure_change(0);
    }
  };

  ADCAdUnit.prototype.on_replay_pressed = function on_replay_pressed() {
    var _this5 = this;

    if (!this.skippable) {
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.close_button_id, false);
    }

    if (this.image_overlay) {

      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.image_overlay_id, true);
    }

    if (this.text_overlay) {

      acni.set_view_visibility("TextView", this.ad_session_id, 1, this.text_overlay_id, true);
    }

    if (this.web_overlay) {

      acni.set_view_visibility("Button", this.ad_session_id, 1, this.overlay_button_id, true);
    }

    if (this.mute_enabled) {
      if (this.muted) {
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, true);
      } else {
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.mute_id, true);
      }
    }

    acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.replay_button_id, false);

    acni.set_view_visibility("WebView", this.ad_session_id, 1, this.web_view_id, false);

    acni.seek_video_to_time(this.video_view_id, this.ad_session_id, 1, 0, function (message_reply) {
      if (lookup_boolean(message_reply, "success")) {
        acni.set_view_visibility("VideoView", _this5.ad_session_id, 1, _this5.video_view_id, true);
        acni.mraid_on_viewable_change(_this5.module_id, _this5.web_view_module_id, false);

        acni.play_video(_this5.video_view_id, _this5.ad_session_id, 1, true);
        _this5.reset_video_tracking();
      }
    });

    this.replay = true;
    acni.ad_unit_on_replay(this.ad_session_id, this.replay, this.v4vc_enabled);

    acni.mraid_on_replay(this.module_id, this.web_view_module_id);
  };

  ADCAdUnit.prototype.on_video_view_ready = function on_video_view_ready(dict) {
    var _this6 = this;

    if (lookup_number(dict, "id") === this.video_view_id || this.os_name === "ios") {

      if (this.muted) {
        acni.set_video_view_volume(this.video_view_id, this.ad_session_id, 1, 0, function (reply) {
          log_info("Video view volume auto muted");
        });
      }

      acni.set_view_visibility("VideoView", this.ad_session_id, 1, this.video_view_id, true);

      if (this.elapsed >= 0) {
        acni.seek_video_to_time(this.video_view_id, this.ad_session_id, 1, this.elapsed, function () {});
      }

      acni.play_video(this.video_view_id, this.ad_session_id, 1);
      if (this.muted) {

        acni.audio_stopped(this.ad_session_id);
      } else if (this.os_name === "ios") {

        acni.audio_started(this.ad_session_id);
      }

      acni.ad_unit_on_start(this.ad_session_id, this.replay, this.v4vc_enabled, this._start);

      if (this.dec_source_path !== "") {

        this.metadata["ad_session_id"] = this.ad_session_id;
        this.metadata["reward"] = this.v4vc_enabled;
        this.metadata["os_name"] = this.os_name;
        this.metadata["ad_unit_module_id"] = this.module_id;
        this.web_view_id = this.next_view_id++;

        var custom_data = {
          "filepath": this.dec_source_path,
          "mraid_filepath": this.mraid_filepath,
          "use_uiwebview": false,
          "enable_messages": true,
          "url": this.html_tag,
          "base_url": this.html_tag,
          "is_module": false,
          "info": this.metadata
        };
        var safe_wv = this.get_safe_webview();
        acni.create_web_view(this.ad_session_id, 1, this.web_view_id, safe_wv.x, safe_wv.y, safe_wv.w, safe_wv.h, custom_data, function (reply) {
          _this6.web_view_module_id = lookup_number(reply, "module_id");
          _this6.dec_load_start = true;
          _this6.dec_load_time = Date.now();
          log_trace("DEC load time: " + _this6.dec_load_time);
        });

        this.replay_button_id = this.next_view_id++;
        var image_data = {
          "filepath": this.button_paths["replay_normal"],
          "autoresizing": 1 << 2 | 1 << 5,
          "is_close_button": false
        };

        var safe_button_pos = this.get_safe_button_position("top_left");
        acni.create_image_view(this.ad_session_id, 1, this.replay_button_id, safe_button_pos.x, safe_button_pos.y, this.button_size, this.button_size, image_data);
      }

      if (this.skippable || this.dec_source_path !== "") {
        this.close_button_id = this.next_view_id++;

        var actual_width = this.screen_width;
        if (this.screen_width < 1000 && this.screen_height < 1000) {
          if (this.orientation === 0) {

            if (this.screen_height < this.screen_width) {
              actual_width = this.screen_height;
            }
          } else if (this.orientation === 1) {

            if (this.screen_height > this.screen_width) {
              actual_width = this.screen_height;
            }
          }
        }

        var image_data = {
          "filepath": this.button_paths["close_normal"],
          "autoresizing": 1 << 0 | 1 << 5,
          "is_close_button": false
        };

        var safe_button_pos = this.get_safe_button_position("top_right", actual_width, this.screen_height);
        acni.create_image_view(this.ad_session_id, 1, this.close_button_id, safe_button_pos.x, safe_button_pos.y, this.button_size, this.button_size, image_data);
        this.close_created = true;
      }

      if (this.image_overlay) {
        this.image_overlay_id = this.next_view_id++;
        var image_data = {
          "dpi": true,
          "filepath": this.image_normal_filepath
        };
        acni.create_image_view(this.ad_session_id, 1, this.image_overlay_id, this.screen_width, this.screen_height, this.engagement_width, this.engagement_height, image_data);
      }

      if (this.text_overlay) {
        this.text_overlay_id = this.next_view_id++;
        var custom_data = {
          "align_x": 0,
          "align_y": 0,
          "font_size": 18,
          "overlay": true,
          "wrap_content": true,
          "text": this.engagement_text,
          "background_color": "FFD3D3D3",
          "font_color": "FF000000",
          "font_family": 0,
          "button": true,
          "editable": false
        };
        acni.create_text_view(this.ad_session_id, 1, this.text_overlay_id, this.screen_width, this.screen_height, this.engagement_width, this.engagement_height, custom_data);
      }

      if (this.web_overlay) {
        var center_x = this.screen_width / 2.0;
        var center_y = this.screen_height / 2.0;
        var origin_x = center_x + this.screen_width / 2.0 - 170;
        var origin_y = center_y + this.screen_height / 2.0 - this.engagement_height;

        this.overlay_button_id = this.next_view_id++;

        var web_content_enabled;
        if (this.engagement_label_html.indexOf('<') === 0) {
          web_content_enabled = true;
        } else {
          web_content_enabled = false;
        }

        if (web_content_enabled) {
          this.overlay_web_view_id = this.next_view_id++;
          var html_content = "<html><head><style type='text/css'>html,body{margin:0;width:100%%;height:100%%;font-family:'helvetica';font-size:28px;white-space:nowrap;}html{display:table;}body{display:table-cell;vertical-align:middle;padding:0px;text-align:center;-webkit-text-size-adjust:none;}</style></head><body>" + this.engagement_label_html + "</body></html>";
        }

        acni.create_button(this.ad_session_id, 1, this.overlay_button_id, origin_x, origin_y, 170, this.engagement_height, this.image_normal_filepath, this.image_down_filepath, web_content_enabled, this.engagement_label_html, html_content, this.overlay_web_view_id);
      }

      if (this.mute_enabled && this.auto_mute) {
        acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, true);
      }

      $.message_dispatcher.register_message_handler(function (message) {
        log_info("WebView.on_load detected");
        if (lookup(message, "url") === _this6.html_tag) {
          _this6.dec_loaded = true;
          _this6.dec_load_time = (Date.now() - _this6.dec_load_time) / 1000;
          log_trace("DEC load time: " + _this6.dec_load_time);
        }
      }, "WebView.on_load");

      $.message_dispatcher.register_message_handler(function (message) {
        log_trace("WebView.on_error detected");
      }, "WebView.on_error");

      log_trace("Loading dec source from " + this.dec_source_path);
    } else {
      acni.play_video(this.self_hosted_video_view_id, this.ad_session_id, 1);
    }
  };

  ADCAdUnit.prototype.on_video_view_progress = function on_video_view_progress(dict) {

    var elapsed = lookup_number(dict, "elapsed");
    var duration = lookup_number(dict, "duration");
    var percent_complete = elapsed / duration;

    var real_percent_complete = elapsed / this.video_duration;
    if (real_percent_complete > .90 && !this.verified_reward) {
      this.verified_reward = true;
      log_info("Reward verified");
    }

    if (this.skippable && elapsed > this.skip_delay && !this.skip_delay_met && this.close_created) {
      this.skip_delay_met = true;
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.close_button_id, true);
    }

    if (this.image_overlay && elapsed > this.engagement_delay && !this.engagement_delay_met) {
      this.engagement_delay_met = true;
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.image_overlay_id, true);
    }

    if (this.text_overlay && elapsed > this.engagement_delay && !this.engagement_delay_met) {
      this.engagement_delay_met = true;
      acni.set_view_visibility("TextView", this.ad_session_id, 1, this.text_overlay_id, true);
    }

    if (this.web_overlay && elapsed > this.engagement_delay && !this.engagement_delay_met) {
      this.engagement_delay_met = true;
      acni.set_view_visibility("Button", this.ad_session_id, 1, this.overlay_button_id, true);
    }

    if (this.mute_enabled && (elapsed >= this.mute_delay || this.auto_mute) && !this.muted && !this.mute_delay_met) {
      this.mute_delay_met = true;
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.mute_id, true);
    }
    if (this.mute_enabled && (elapsed >= this.mute_delay || this.auto_mute) && this.muted && !this.unmute_delay_met) {
      this.unmute_delay_met = true;
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, true);
    }

    if (!this.video_finished && !this.self_hosted_video_playing) {
      if (percent_complete >= .25 && !this.first_quartile_sent) {
        this.on_video_first_quartile(dict);
      } else if (percent_complete >= .50 && !this.midpoint_sent) {
        this.on_video_midpoint(dict);
      } else if (percent_complete >= .75 && !this.third_quartile_sent) {
        this.on_video_third_quartile(dict);
      } else if (elapsed === duration && !this.video_finished && duration !== 0) {
        this.on_video_finished(dict, true);
      }
    } else if (elapsed === duration && this.os_name === "android") {
      if (this.self_hosted_video_playing) {
        acni.destroy_view("VideoView", this.ad_session_id, 1, this.self_hosted_video_view_id);
        acni.destroy_view("ColorView", this.ad_session_id, 1, this.color_view_id);
        this.self_hosted_video_playing = false;
      }
    }
  };

  ADCAdUnit.prototype.on_video_first_quartile = function on_video_first_quartile(message) {
    log_trace("Video view first quartile met");
    this.first_quartile_sent = true;

    acni.ad_unit_on_video_first_quartile(this.ad_session_id, this.replay, this.v4vc_enabled);
  };

  ADCAdUnit.prototype.on_video_midpoint = function on_video_midpoint(message) {
    log_trace("Video view midpoint met");
    this.midpoint_sent = true;

    acni.ad_unit_on_video_midpoint(this.ad_session_id, this.replay, this.v4vc_enabled);
  };

  ADCAdUnit.prototype.on_video_third_quartile = function on_video_third_quartile(message) {
    log_trace("Video view third quartile met");
    this.third_quartile_sent = true;

    acni.ad_unit_on_video_third_quartile(this.ad_session_id, this.replay, this.v4vc_enabled);
  };

  ADCAdUnit.prototype.reset_video_tracking = function reset_video_tracking() {
    this.first_quartile_sent = false;
    this.midpoint_sent = false;
    this.third_quartile_sent = false;
    this.video_finished = false;
    this.dissolve_time = 0;
    this.current_viewable_percentage_time = 0;
    this.last_viewability_update = Date.now();
    this.on_viewable_impression_called = false;
  };

  ADCAdUnit.prototype.on_video_finished = function on_video_finished(message, finished) {
    log_trace("Video view finished");
    this.video_finished = true;

    if (finished) {
      acni.ad_unit_on_video_finish(this.ad_session_id, this.replay, this.v4vc_enabled);
    }

    if (this.image_overlay) {
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.image_overlay_id, false);
    } else if (this.text_overlay) {
      acni.set_view_visibility("TextView", this.ad_session_id, 1, this.text_overlay_id, false);
    } else if (this.web_overlay) {
      acni.set_view_visibility("Button", this.ad_session_id, 1, this.overlay_button_id, false);
    }

    if (this.dec_loaded) {
      log_info("Web view loaded");

      acni.set_view_visibility("VideoView", this.ad_session_id, 1, this.video_view_id, false);

      acni.set_view_visibility("WebView", this.ad_session_id, 1, this.web_view_id, true);

      this.dec_time_spent = Date.now();

      acni.mraid_on_viewable_change(this.module_id, this.web_view_module_id, true);
      this.on_card_shown(message);

      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.close_button_id, true);
      acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.replay_button_id, true);

      this.replay_wait_time = Date.now() + 0;

      acni.move_view_to_front(this.ad_session_id, 1, this.replay_button_id);
      acni.move_view_to_front(this.ad_session_id, 1, this.close_button_id);

      if (this.mute_enabled) {
        if (!this.muted) {
          acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.mute_id, false);
        } else {
          acni.set_view_visibility("ImageView", this.ad_session_id, 1, this.unmute_id, false);
        }
      }

      var view_id = -1;
      if (this.image_overlay) {
        view_id = this.image_overlay_id;
      } else if (this.web_overlay) {
        view_id = this.overlay_button_id;
      } else if (this.text_overlay) {
        view_id = this.text_overlay_id;
      }
      if (view_id !== -1) {
        acni.move_view_to_front(this.ad_session_id, 1, view_id);
      }

      return;
    }

    log_info("web view not loaded");

    this.dec_load_timeout = true;

    this.finish_ad(false);
  };

  ADCAdUnit.prototype.on_exposure_change = function on_exposure_change(percentage) {

    if (!this.viewability_enabled || this.current_viewable_percentage == percentage) {
      return;
    }

    if (this.showing_alert) {
      percentage = 0;
    }

    log_info("Exposure change detected: " + percentage);

    this.current_viewable_percentage = percentage;

    if (this.current_viewable_percentage < this.viewable_percentage) {
      this.current_viewable_percentage_time = 0;
    }
  };

  ADCAdUnit.prototype.check_viewable_impression = function check_viewable_impression() {
    log_info("check_viewable_impression called");

    if (this.current_viewable_percentage >= this.viewable_percentage) {
      this.current_viewable_percentage_time += (Date.now() - this.last_viewability_update) / 1000;
      log_info("Current viewable percentage time: " + this.current_viewable_percentage_time);
      log_info("Viewable duration: " + this.viewable_duration);
    }

    if (this.current_viewable_percentage_time >= this.viewable_duration && this.current_viewable_percentage >= this.viewable_percentage) {
      log_info("Detecting viewable impression");
      this.on_viewable_impression();
    }

    this.last_viewability_update = Date.now();
  };

  ADCAdUnit.prototype.on_viewable_impression = function on_viewable_impression() {

    var custom_data = {
      "replay": this.replay,
      "reward": this.v4vc_enabled,
      "event_type": "viewable_impression"
    };

    log_info("on_viewable_impression called");

    acni.on_ad_unit_event(this.ad_session_id, custom_data);

    this.on_viewable_impression_called = true;
  };

  ADCAdUnit.prototype.get_viewable_impression_called = function get_viewable_impression_called() {
    return this.on_viewable_impression_called;
  };

  ADCAdUnit.prototype.is_viewability_enabled = function is_viewability_enabled() {
    return this.viewability_enabled;
  };

  ADCAdUnit.prototype.finish_ad = function finish_ad(error) {
    var _this7 = this;

    if (this.finishing) {
      return;
    }

    this.finishing = true;

    var status;
    if (error) {
      status = 1;
    } else {
      status = this.video_finished || this.replay ? 0 : 6;
    }

    if (this.skipped_to_dec && !this.dec_loaded) {
      status = 6;
    }

    if (this.expanded !== -1) {
      acni.ad_unit_on_expand(this.module_id, this.expanded);
    }

    if (!this.muted) {
      acni.audio_stopped(this.ad_session_id);
    }

    if (this.video_finished || this.replay || lookup(this.v4iap, "engagement_type") !== 'undefined') {
      this.v4vc_videos_watched++;
    }

    if (this.client_side && (!this.skipped_to_dec && (this.video_finished || this.replay) || lookup(this.v4iap, "engagement_type") !== 'undefined')) {
      if (this.v4vc_videos_watched >= this.views_per_reward) {
        var custom_data = {
          "reward_amount": this.reward_amount,
          "reward_name": this.reward_name,
          "zone_id": this.zone_id
        };
        acni.ad_on_reward_v4vc(this.ad_session_id, this.replay, true, custom_data);
      }
    }

    if (this.video_finished || this.replay || lookup(this.v4iap, "engagement_type") !== 'undefined') {
      this.reward_v4vc(true);
    } else {
      this.reward_v4vc(false);
    }

    if (this.dec_loaded && this.dec_time_spent !== 0) {
      this.dec_time_spent = (Date.now() - this.dec_time_spent) / 1000;
      log_trace("DEC time spent: " + this.dec_time_spent);
    }

    var v4iap = this.v4iap_enabled && lookup(this.v4iap, "engagement_type") !== 'undefined' ? this.v4iap : undefined;
    acni.finish_fullscreen_ad(this.ad_session_id, status, 1, v4iap, function (reply) {
      var post_popup = !_this7.skipped_to_dec && (_this7.video_finished || _this7.replay) || lookup(_this7.v4iap, "engagement_type") !== 'undefined' ? _this7.post_popup : false;
      acni.ad_unit_on_finish(_this7.zone_id, _this7.ad_session_id, status, post_popup, _this7.replay, _this7.v4vc_enabled, _this7.dec_load_start, _this7.dec_loaded, _this7.dec_load_time, _this7.dec_load_timeout, _this7.dec_time_spent, _this7.dissolved, !_this7.skipped_to_dec && _this7.video_finished, _this7.skip_type, lookup(_this7.v4iap, "engagement_type") !== 'undefined', function (reply) {
        _this7.finishing = false;

        if (!_this7.from_native) {
          _this7.destroy_session();
        }

        _this7.destroy_module();
      });
    });

    if (this.v4vc_videos_watched >= this.views_per_reward) {
      this.v4vc_videos_watched = 0;
    }
  };

  ADCAdUnit.prototype.reward_v4vc = function reward_v4vc(success) {

    success = success && (this.verified_reward && !this.skipped_to_dec || this.v4iap_enabled && lookup(this.v4iap, "engagement_type") !== "undefined");
    if (this.v4vc_enabled && this.client_side || this.v4vc_enabled && !this.client_side && !success) {
      var isRewardSuccessful = false;
      if (success && this.v4vc_videos_watched >= this.views_per_reward) {
        isRewardSuccessful = true;
      }
      acni.v4vc_reward(isRewardSuccessful, this.reward_amount, this.reward_name, this.zone_id);
    } else if (this.v4vc_enabled && !this.client_side && success) {
      if (this.v4vc_videos_watched >= this.views_per_reward) {
        acni.ad_unit_on_v4vc_request(this.ad_session_id, this.zone_id, this.replay, this.reward_amount, this.reward_name);
      }
    }
  };

  ADCAdUnit.prototype.destroy_session = function destroy_session() {
    acni.destroy_ad_session(this.ad_session_id);
  };

  ADCAdUnit.prototype.destroy_module = function destroy_module() {
    acni.unload_module(this.ad_session_id);
  };

  ADCAdUnit.prototype.get_safe_button_position = function get_safe_button_position(pos, screen_width, screen_height) {
    var button_x = 0;
    var button_y = 0;

    screen_width = screen_width || this.screen_width;
    screen_height = screen_height || this.screen_height;

    if (pos == "top_left") {
      button_x = this.safe_inset_left;
      button_y = 0;
      if (screen_width < screen_height) {
        button_x = 0;
        button_y = this.safe_inset_top;
      }
    } else if (pos == "top_right") {
      button_x = screen_width - this.button_size - this.safe_inset_right;
      button_y = 0;
      if (screen_width < screen_height) {
        button_x = screen_width - this.button_size;
        button_y = this.safe_inset_top;
      }
    } else if (pos == "bottom_right") {
      button_x = screen_width - this.button_size - this.safe_inset_right;
      button_y = screen_height - this.button_size - this.safe_inset_bottom;
    }

    return { x: button_x, y: button_y };
  };

  ADCAdUnit.prototype.get_safe_webview = function get_safe_webview(screen_width, screen_height) {
    screen_width = screen_width || this.screen_width;
    screen_height = screen_height || this.screen_height;

    var safe_x = 0;
    var safe_y = 0;
    var safe_width = screen_width;
    var safe_height = screen_height;

    if (screen_width < screen_height) {
      safe_y = this.safe_inset_top;
      safe_height = screen_height - this.safe_inset_top - this.safe_inset_bottom;
    } else {
      safe_x = this.safe_inset_left;
      safe_width = screen_width - this.safe_inset_left - this.safe_inset_right;
      safe_height = screen_height - this.safe_inset_bottom;
    }

    return { x: safe_x, y: safe_y, w: safe_width, h: safe_height };
  };

  ADCAdUnit.prototype.position_top_right = function position_top_right() {};

  ADCAdUnit.prototype.position_bottom_right = function position_bottom_right() {};

  return ADCAdUnit;
})();

var $ = {
  enabled: true,
  ad_unit: null,
  message_dispatcher: null,
  log: null
};
var ADC3_init = function ADC3_init(module_id, ad_unit_config) {
  if (typeof ad_unit_config == 'string') {
    ad_unit_config = JSON.parse(ad_unit_config);
  }

  var os_name = lookup_string(ad_unit_config, "os_name");

  $.enabled = true;
  $.message_dispatcher = new ADCMessageDispatcher();
  $.log = new ADCLogger($.message_dispatcher, true, os_name);
  $.ad_unit = new ADCAdUnit(module_id, ad_unit_config);

  $.ad_unit.start();
};

var ADC3_update = function ADC3_update(inbound_messages) {
  if (typeof inbound_messages === 'string') {
    inbound_messages = JSON.parse(inbound_messages);
  }

  if (!$.ad_unit.get_viewable_impression_called() && $.ad_unit.is_viewability_enabled()) {
    $.ad_unit.check_viewable_impression();
  }

  if (!$.ad_unit.finishing && $.ad_unit.video_finished && $.ad_unit.dissolve) {
    if ($.ad_unit.dissolve_time === 0) {
      $.ad_unit.dissolve_time = Math.floor(new Date().getTime() / 1000);
    }

    if (Math.floor(new Date().getTime() / 1000 - $.ad_unit.dissolve_time > $.ad_unit.dissolve_delay) && !$.ad_unit.dissolved) {
      $.ad_unit.dissolved = true;
      $.ad_unit.finish_ad(false);

      var custom_data = {
        "replay": $.ad_unit.replay,
        "reward": $.ad_unit.v4vc_enabled,
        "event_type": "card_dissolved"
      };

      acni.on_ad_unit_event($.ad_unit.ad_session_id, custom_data);
    }
  }

  $.message_dispatcher.distribute(inbound_messages);
  var outbound = $.message_dispatcher["package"]();
  return outbound;
};

var ADC3_on_button_pressed = function ADC3_on_button_pressed(view_id) {
  $.ad_unit.on_button_pressed(view_id);
};

var ADC3_on_video_view_ready = function ADC3_on_video_view_ready(dict) {
  $.ad_unit.on_video_view_ready(dict);
};

var ADC3_on_video_view_progress = function ADC3_on_video_view_progress(dict) {
  $.ad_unit.on_video_view_progress(dict);
};

var ADC3_on_container_touch_began = function ADC3_on_container_touch_began(dict) {
  $.ad_unit.on_container_touch_began(dict);
};

var ADC3_on_container_touch_moved = function ADC3_on_container_touch_moved(dict) {
  $.ad_unit.on_container_touch_moved(dict);
};

var ADC3_on_container_touch_ended = function ADC3_on_container_touch_ended(dict) {
  $.ad_unit.on_container_touch_ended(dict);
};

var ADC3_on_container_touch_cancelled = function ADC3_on_container_touch_cancelled(dict) {
  $.ad_unit.on_container_touch_cancelled(dict);
};

var ADC3_on_container_resized = function ADC3_on_container_resized(dict) {
  $.ad_unit.on_container_resized(dict);
};

var ADC3_on_store_kit_opened = function ADC3_on_store_kit_opened() {
  $.ad_unit.on_store_kit_opened();
};

var ADC3_on_store_kit_closed = function ADC3_on_store_kit_closed() {
  $.ad_unit.on_store_kit_closed();
};

var ADC3_on_exposure_change = function ADC3_on_exposure_change(percentage) {
  $.ad_unit.on_exposure_change(percentage);
};
